@@ -71,6 +71,10 @@ class VMResource(BaseResource):
         super().__init__(params)
         # mark the resource created, default is os_tests_vm
         self.tag = 'os_tests_vm'
+        # dead_count to monitor resource health status
+        # for example, if the vm cannot connect over X times, we can skip the follow cases
+        # no need to retry connection until timeout
+        self.dead_count = 0
 
     @property
     @abstractmethod
@@ -86,6 +90,22 @@ class VMResource(BaseResource):
         :return: vm's ipv6 or FQDN
         """
 
+    @property
+    def is_secure_boot(self):
+        """
+        vm provisioned with secure boot or not
+        :return: True|False
+        """
+        return False
+
+    @property
+    def is_uefi_boot(self):
+        """
+        vm provisioned with uefi or not
+        :return: True|False
+        """
+        return False
+
     @property
     def port(self):
         """
@@ -94,6 +114,13 @@ class VMResource(BaseResource):
         """
         return 22
 
+    @property
+    def is_metal(self):
+        """
+        return if vm is metal or not
+        """
+        return False
+
     @property
     @abstractmethod
     def disk_count(self):
