@@ -3,6 +3,7 @@ from os_tests.libs import utils_lib
 import logging
 import time
 import sys
+import re
 try:
     import boto3
     from botocore.exceptions import ClientError
@@ -27,7 +28,7 @@ class EC2VM(VMResource):
         self.session = boto3.session.Session(profile_name=self.profile_name, region_name=params.get('region'))
         self.resource = self.session.resource('ec2', config=config)
         self.client = self.session.client('ec2', config=config, region_name=params.get('region'))
-        super(EC2VM, self).__init__(params)
+        super().__init__(params)
         self.id = None
         self.ipv4 = None
         self.ssh_user = params.get('remote_user')
@@ -71,16 +72,20 @@ class EC2VM(VMResource):
         self.is_created = False
         self.another_ip = None
         self.run_uuid = params.get('run_uuid')
+        self.user_data = '#!/bin/bash\nmkdir -p /tmp/userdata_{}'.format(self.run_uuid)
+        self.hibernation_support = False
+        # efa_support default set to False, will query instance property next
+        self.efa_support = False
 
     def show(self):
         if self.is_exist():
             LOG.info("Instance ID: {}".format(self.ec2_instance.id))
 
-    def create(self, wait=True):
+    def create(self, wait=True, enable_efa=True):
+        # enable_efa is option to enable or disable efa when create vms
+        # if vm does not support efa, it will be disabled
         self.is_created = False
-        # start with efa enabled if it is supported
-        self.efa_support = False
-        self.hibernation_support = False
+
         try:
             self.efa_support = self.client.describe_instance_types(
                 InstanceTypes=[self.instance_type],
@@ -102,13 +107,14 @@ class EC2VM(VMResource):
         except Exception as error:
             LOG.info('Cannot determin root device name, use default {}'.format(self.root_device_name))
 
+        volume_size = 10
         vm_kwargs = {
             'BlockDeviceMappings':[
                 {
                     'DeviceName': self.root_device_name,
                     'Ebs': {
                         'DeleteOnTermination': True,
-                        'VolumeSize': 10,
+                        'VolumeSize': volume_size,
                         # root disk must be encrypted when hibernation enabled
                         'Encrypted': self.hibernation_support
                     },
@@ -144,34 +150,55 @@ class EC2VM(VMResource):
             'HibernationOptions':{
                 'Configured': self.hibernation_support
             },
-            "UserData":'#!/bin/bash\nmkdir /tmp/userdata_{}'.format(self.run_uuid)
+            "UserData":self.user_data
         }
         if self.efa_support:
-            vm_kwargs["NetworkInterfaces"][0]["InterfaceType"] = 'efa'
-        #vm_kwargs["EnclaveOptions"]["Enabled"] = True
-        if self.additionalinfo == None or self.additionalinfo == '':
-            try:
-                self.ec2_instance = self.resource.create_instances(**vm_kwargs)[0]
-                self.is_created = True
-            except ClientError as err:
-                LOG.error("Failed to create instance!")
-                self.is_createable = False
-                raise err
-            except Exception as err:
-                raise err
-        if self.additionalinfo != None and self.additionalinfo != '':
-            for additionalinfo in self.additionalinfo.split(';'):
+            if enable_efa:
+                vm_kwargs["NetworkInterfaces"][0]["InterfaceType"] = 'efa'
+            else:
+                LOG.info("efa is supported, but disable it as request")
+        #vm_kwargs["EnclaveOptions"]["Enabled"] = True       
+        if not self.additionalinfo:
+            for volume_size in [10,20,40,50]:
+                LOG.info("Create instance {}".format(vm_kwargs))
                 try:
-                    LOG.error("Create instance with AdditionalInfo: {}".format(additionalinfo))
-                    vm_kwargs['AdditionalInfo'] = additionalinfo
                     self.ec2_instance = self.resource.create_instances(**vm_kwargs)[0]
                     self.is_created = True
+                    break
                 except Exception as err:
-                    LOG.error("Failed to create instance, try another AdditionalInfo {}".format(err))
+                    LOG.error("Failed to create instance with error:{}".format(err))
+                    if 'UnsupportedHibernationConfiguration' in str(err):
+                        vm_kwargs["BlockDeviceMappings"][0]['Ebs']['VolumeSize'] = volume_size
+                        LOG.info("Increase disk size {}".format(volume_size))
+                        if volume_size == 40:
+                            LOG.info("try to launch with hibernation disabled")
+                            self.hibernation_support = False
+                            vm_kwargs["HibernationOptions"]['Configured'] = self.hibernation_support
+                        continue
+        if self.additionalinfo:
+            for additionalinfo in self.additionalinfo.split(';'):
+                LOG.info("try addtionalinfo:{}".format(additionalinfo))
+                vm_kwargs['AdditionalInfo'] = additionalinfo
+                for volume_size in [10,20,40,50]:
+                    LOG.info("Create instance {}".format(vm_kwargs))
+                    try:
+                        self.ec2_instance = self.resource.create_instances(**vm_kwargs)[0]
+                        self.is_created = True
+                        break
+                    except Exception as err:
+                        LOG.error("Failed to create instance with error:{}".format(err))
+                        if 'UnsupportedHibernationConfiguration' in str(err):
+                            vm_kwargs["BlockDeviceMappings"][0]['Ebs']['VolumeSize'] = volume_size
+                            LOG.info("Increase disk size {}".format(volume_size))
+                            if volume_size == 40:
+                                LOG.info("try to launch with hibernation disabled")
+                                self.hibernation_support = False
+                                vm_kwargs["HibernationOptions"]['Configured'] = self.hibernation_support
+                            continue
                 if self.is_created:
                     break
-            if not self.is_created:
-                raise 'Cannot create'
+        if not self.is_created:
+            raise Exception("Cannot create instance")
         if wait:
             try:
                 self.ec2_instance.wait_until_running()
@@ -196,6 +223,21 @@ class EC2VM(VMResource):
         LOG.info("Public ip is: %s" % self.ipv4)
         return self.ipv4
 
+    @property
+    @utils_lib.wait_for(not_ret=None, ck_not_ret=True, timeout=120)
+    def private_ip(self):
+        self.ec2_instance.reload()
+        LOG.info("private_ip_address is: {}".format(self.ec2_instance.private_ip_address))
+        return self.ec2_instance.private_ip_address
+
+    @property
+    @utils_lib.wait_for(not_ret=None, ck_not_ret=True, timeout=120)
+    def vm_name(self):
+        self.ec2_instance.reload()
+        LOG.info("private_dns_name is: {}".format(self.ec2_instance.private_dns_name))
+        return self.ec2_instance.private_dns_name
+
+
     @property
     @utils_lib.wait_for(not_ret=None, ck_not_ret=True, timeout=120)
     def ipv6_address(self):
@@ -206,12 +248,64 @@ class EC2VM(VMResource):
 
     @property
     def disk_count(self):
-        volumes_list = []
+        default_disks = 1
+        instance_types_dict = self.client.describe_instance_types(InstanceTypes=[self.instance_type])
+        instance_types = instance_types_dict.get('InstanceTypes')
+        if not instance_types:
+            LOG.info("Cannot get instance type info, return {}".format(default_disks))
+            return default_disks
+        for instance_type in instance_types:
+            if instance_type['InstanceStorageSupported']:
+                default_disks = instance_type["InstanceStorageInfo"]["Disks"][0]["Count"] + 1
+        LOG.info("{} has {} disks".format(self.instance_type, default_disks))
+        return default_disks
+
+    @property
+    def net_bandwidth(self):
+        default_bandwidth = 5
+        instance_types_dict = self.client.describe_instance_types(InstanceTypes=[self.instance_type])
+        instance_types = instance_types_dict.get('InstanceTypes')
+        if not instance_types:
+            LOG.info("Cannot get instance type info, return default bandwidth {}".format(default_bandwidth))
+            return default_bandwidth
+        for instance_type in instance_types:
+            if instance_type.get('NetworkInfo'):
+                net_bandwidth = instance_type["NetworkInfo"]["NetworkPerformance"]
+        if 'Gigabit' in net_bandwidth:
+            net_bandwidth = re.findall('[\d]+', net_bandwidth)[0]
+        else:
+            LOG.info("{} net setting spec {}, ret {}".format(self.instance_type, net_bandwidth, default_bandwidth))
+            return default_bandwidth
+        LOG.info("{} net setting spec {}".format(self.instance_type, net_bandwidth))
+        return int(net_bandwidth)
+
+    @property
+    def is_secure_boot(self):
+        return False
+
+    @property
+    def is_uefi_boot(self):
         self.ec2_instance.reload()
-        for i in self.ec2_instance.volumes.all():
-            volumes_list.append(i.id)
-        LOG.info(volumes_list)
-        return len(volumes_list)
+        try:
+            bootmode = self.client.describe_instances(
+                InstanceIds=[
+                    self.ec2_instance.id,
+                ])['Reservations'][0]['Instances'][0].get("BootMode")
+            LOG.info('boot mode: {}'.format(bootmode))
+            if bootmode and 'uefi' in bootmode:
+                return True
+            else:
+                return False
+        except Exception as error:
+            LOG.info('Cannot determin boot mode, return False')
+        return False
+
+    @property
+    def is_metal(self):
+        """
+        return if vm is metal or not
+        """
+        return 'metal' in self.instance_type
 
     def start(self, wait=True):
         start_ok = False
@@ -356,6 +450,9 @@ class EC2VM(VMResource):
         except Exception as exc:
             return False
 
+    def exists(self):
+        return self.is_exist()
+
     def is_stopped(self):
         if self.get_state() == 'stopped':
             return True
@@ -404,6 +501,41 @@ class EC2VM(VMResource):
     def detach_nic(self,nic,force=False):
         return nic.detach_from_instance(self.id, wait=True, force=force)
 
+    @property
+    def primary_nic_id(self):
+        for nic in self.ec2_instance.network_interfaces_attribute:
+            if nic['Attachment']['DeviceIndex'] == 0:
+                return nic['NetworkInterfaceId']
+        LOG.info("primary nic id not found")
+        return None
+
+    def assign_new_ip(self):
+        nic = self.resource.NetworkInterface(self.primary_nic_id)
+        ret = nic.assign_private_ip_addresses(
+                AllowReassignment=True,
+                SecondaryPrivateIpAddressCount=1
+            )
+        self.another_ip = ret['AssignedPrivateIpAddresses'][0]['PrivateIpAddress']
+        LOG.info("second nic ip{}".format(self.another_ip))
+        return self.another_ip
+    
+    def remove_added_ip(self):
+        nic = self.resource.NetworkInterface(self.primary_nic_id)
+        if self.another_ip is None:
+            LOG.info("second nic ip is {}".format(self.another_ip))
+            return False
+        try:
+            ret = nic.unassign_private_ip_addresses(
+                    PrivateIpAddresses=[
+                        self.another_ip,
+                    ]
+                )
+            LOG.info("removed second nic ip{}".format(self.another_ip))
+            return True
+        except Exception as err:
+            LOG.info(err)
+        return False
+
 class EC2Volume(StorageResource):
     '''
     Volume class
@@ -561,11 +693,15 @@ class EC2Volume(StorageResource):
             LOG.info("Volume does not exists %s" % self.id)
             return False
 
+    def modify_disk_size(self, os_disk_size, expand_num):
+        raise NotImplementedError
+
 class EC2NIC(NetworkResource):
     '''
     AWS Network class
     '''
     __network_interface = None
+    EC2_CLIENT = boto3.client('ec2')
 
     def __init__(self, params):
         super(EC2NIC, self).__init__(params)
@@ -672,19 +808,31 @@ class EC2NIC(NetworkResource):
             LOG.info("Failed to add tag to %s", self.__network_interface.id)
             LOG.error(err)
 
-    def create(self):
+    def create(self,interfaceType='interface'):
         '''Create a new network interface
         '''
         try:
-            self.__network_interface = self.subnet.create_network_interface(
-                Description=self.tag, Groups=[
-                    self.security_group_ids,
-                ])
-            LOG.info("%s network interface created!" %
-                     self.__network_interface.id)
-            self.add_tag()
-            self.id = self.__network_interface.id
-            return True
+            if interfaceType == 'efa':
+                self.__network_interface = self.subnet.create_network_interface(
+                    Description=self.tag,
+                    InterfaceType='efa',
+                    SubnetId=self.subnet_id,
+                    Groups=[self.security_group_ids, ])
+                LOG.info("%s efa network interface created!" %
+                         self.__network_interface.id)
+                self.add_tag()
+                self.id = self.__network_interface.id
+                return self.__network_interface
+            else:
+                self.__network_interface = self.subnet.create_network_interface(
+                    Description=self.tag, Groups=[
+                        self.security_group_ids,
+                    ])
+                LOG.info("%s network interface created!" %
+                         self.__network_interface.id)
+                self.add_tag()
+                self.id = self.__network_interface.id
+                return True
         except Exception as err:
             LOG.info("Failed to create interface")
             LOG.error(err)
@@ -786,3 +934,36 @@ class EC2NIC(NetworkResource):
             LOG.error("NIC cannot detach from %s error %s" %
                       (instance_id, err))
             return False
+
+    def allocate_eip(self):
+        try:
+            return self.EC2_CLIENT.allocate_address(Domain='vpc', TagSpecifications=[
+                {'ResourceType': 'elastic-ip', 'Tags': [{'Key': 'Name', 'Value': 'efa-elastic-ip'}, ]}, ])
+        except Exception as err:
+            LOG.info("Failed to allocate elastic ip")
+            LOG.error(err)
+
+    def associate_eip(self,instance_id):
+        try:
+            response = self.EC2_CLIENT.describe_addresses(Filters=[{'Name': 'tag:Name', 'Values': ['efa-elastic-ip']}])
+            public_ip = response['Addresses'][0]['PublicIp']
+            allocation_id = response['Addresses'][0]['AllocationId']
+            self.EC2_CLIENT.associate_address(InstanceId=instance_id, AllocationId=allocation_id)
+            LOG.info(f'EIP {public_ip} associated with the instance {instance_id}')
+            return True
+        except Exception as err:
+            LOG.info("Failed to associate elastic ip to the instance")
+            LOG.error(err)
+            return False
+
+    def release_eip(self):
+        try:
+            response = self.EC2_CLIENT.describe_addresses(
+                Filters=[{'Name': 'tag:Name', 'Values': ['efa-elastic-ip']}])
+            public_ip = response['Addresses'][0]['PublicIp']
+            allocation_id = response['Addresses'][0]['AllocationId']
+            self.EC2_CLIENT.release_address(AllocationId=allocation_id)
+            LOG.info(f'EIP {public_ip} has been released')
+        except Exception as err:
+            LOG.info("Failed to release Elastic IP")
+            LOG.error(err)
