@@ -124,7 +124,13 @@ class PrismApi(PrismSession):
         self.memory = params['Flavor']['memory']
         #self.vm_user_data = params.get('custom_data', '*/VM/*')
         self.vm_user_data = params['VM']['custom_data']
+        self.if_uefi_boot = params['VM']['if_uefi_boot']
+        self.if_secure_boot = params['VM']['if_secure_boot']
+        self.machine_type = params['VM']['machine_type']
         self.vm_custom_file = None
+        self.attach_disk_size = 5
+        self.run_uuid = params.get('run_uuid')
+        self.user_data = None
 
         super(PrismApi, self).__init__(self.cvmIP, username, password)
 
@@ -146,9 +152,11 @@ class PrismApi(PrismSession):
             exit(self.r.status_code)
         return json_obj
 
-    def create_vm(self, ssh_pubkey=None):
-        logging.debug("Create VM")
+    def create_vm(self, ssh_pubkey=None, single_nic=True, vm_name=None):
+        logging.info("Create VM, single nis is "+ str(single_nic))
         endpoint = urljoin(self.base_url, "vms")
+        if vm_name == None:
+            vm_name = self.vm_name
 	# Attach image.
         images = self.list_images()
         vmdisk_uuid = ""
@@ -166,32 +174,48 @@ class PrismApi(PrismSession):
             ssh_key = '\nssh_authorized_keys:\n- %s' % ssh_pubkey
             ssh_pwauth = ''
         # Attach user_data.
-        user_data = '#cloud-config\ndisable_root: false\nlock_passwd: false%s%s\n' % (
+        user_data = '#cloud-config\n'
+        user_data_ssh_key = '''\
+disable_root: false
+lock_passwd: false%s%s
+runcmd:
+- grubby --update-kernel=ALL --args="console=tty0"
+- systemctl restart cloud-final
+- sed -i "/PermitRootLogin prohibit/c\PermitRootLogin yes" /etc/ssh/sshd_config
+- systemctl restart sshd\n''' % (
             ssh_pwauth, ssh_key)
+        user_data += user_data_ssh_key+'- mkdir /tmp/userdata_{}\n'.format(self.run_uuid)
+        user_data += '''- [ sh, -xc, "echo $(date) ': hello today!'" ]'''
         if self.vm_user_data:
             user_data += self.vm_user_data
+        if self.user_data != None:
+            user_data = self.user_data
+            user_data += user_data_ssh_key
+        logging.info('userdata for creating vm is \n %s' % user_data)
         # Attach user script.
         user_script=[]
         if self.vm_custom_file:
             user_script = [{'source_path': 'adsf:///{}/{}'.format(self.get_container()['name'], self.vm_custom_file),
                       'destination_path': '/tmp/{}'.format(self.vm_custom_file)}]
-        print(user_script)
-        # Attach NICs (all).
-        network_uuids = []
-        for network in self.list_networks_detail()["entities"]:
-            network_uuids.append({"network_uuid": network["uuid"]})
+        # Attach NICs (all).--> Change to attach specified uuid unless test_persistent_route will fail
+        network_uuids = [{"network_uuid": self.network_uuid}]
+        if not single_nic:
+            for network in self.list_networks_detail()["entities"]:
+                if network["uuid"] != self.network_uuid:
+                    network_uuids.append({"network_uuid": network["uuid"]})
         data = {'boot': {
                 'disk_address': {
                 'device_bus': 'scsi',
                 'device_index': 0
                 },
                 'boot_device_type': 'disk',
-                'uefi_boot': False
+                'uefi_boot': self.if_uefi_boot,
+                'secure_boot': self.if_secure_boot
 	        },
             'memory_mb':
             self.memory * 1024,
             'name':
-            self.vm_name,
+            vm_name,
             'num_cores_per_vcpu':
             1,
             'num_vcpus':
@@ -226,16 +250,19 @@ class PrismApi(PrismSession):
                 "is_cdrom": False,
                 "is_empty": True,
                 "vm_disk_create": {
-                "size": 1*1024*1024*1024,
-                "storage_container_uuid": "41797d39-e961-4ab5-adcc-e5c9ab817729"
+                "size": self.attach_disk_size*1024*1024*1024,
+                "storage_container_uuid": self.storage_container_uuid
                 }
             }
         ],
-            'vm_nics': network_uuids
+            'vm_nics': network_uuids,
+            'machine_type': self.machine_type
         }
         return self.make_request(endpoint, 'post', data=data)
-       
-    def create_vm_ISO_kickstart(self):
+
+    def create_vm_ISO_kickstart(self, single_nic=True, vm_name=None):
+        if not vm_name:
+            vm_name = self.vm_name + vm_name
         logging.debug("Create VM by ISO kickstart")
         endpoint = urljoin(self.base_url, "vms")
 	# Attach image.
@@ -250,9 +277,11 @@ class PrismApi(PrismSession):
             logging.error("Image not found, image list be got is %s" % str(vmdisk_uuid))
             exit(1)
         # Attach NICs (all).
-        network_uuids = []
-        for network in self.list_networks_detail()["entities"]:
-            network_uuids.append({"network_uuid": network["uuid"]})
+        network_uuids = [{"network_uuid": self.network_uuid}]
+        if not single_nic:
+            for network in self.list_networks_detail()["entities"]:
+                if network["uuid"] != self.network_uuid:
+                    network_uuids.append({"network_uuid": network["uuid"]})
         data = {
             'boot': {
                 'uefi_boot': False
@@ -261,7 +290,7 @@ class PrismApi(PrismSession):
             'memory_mb':
             self.memory * 1024,
             'name':
-            self.vm_name,
+            vm_name,
             'num_cores_per_vcpu':
             1,
             'num_vcpus':
@@ -330,7 +359,7 @@ class PrismApi(PrismSession):
         endpoint = urljoin(self.base_url, "vms/%s/set_power_state" % vm_uuid)
         data = {"transition": "ON"}
         return self.make_request(endpoint, 'post', data=data)
-        
+    
     def migrate_vm(self, vm_uuid, host_uuid=None):
         # Use API v0.8 for migrate operation
         logging.info("Migrate VM")
@@ -379,6 +408,13 @@ class PrismApi(PrismSession):
             "networks/")
         return self.make_request(endpoint, 'get')
 
+    def list_networks_address(self, network_uuid):
+        logging.debug("Get networks address by networks uuid")
+        endpoint = urljoin(
+            self.base_url,
+            "networks/%s/addresses" % network_uuid)
+        return self.make_request(endpoint, 'get')
+
     def update_vcpu(self, vm_uuid, vcpu_num):
         logging.debug("Update vCPU number")
         endpoint = urljoin(self.base_url, "vms/%s" % vm_uuid)
@@ -429,17 +465,44 @@ class PrismApi(PrismSession):
                 endpoint = urljoin(self.base_url, "networks/%s" % network["uuid"])
                 self.make_request(endpoint, 'delete')
 
-    def attach_disk(self, vm_uuid, device_bus, disk_size, is_cdrom, device_index, *is_empty):
+    def attach_nic(self, vm_uuid, network_uuid, ip_address=None, driver='virtio'):
+        logging.info("Attach nic to vm")
+        endpoint = urljoin(self.base_url, "vms/%s/nics/" % vm_uuid)
+        if ip_address==None:
+            data = {'spec_list': [{
+                    'is_connected': True,
+                    'model':driver,
+                    'network_uuid': network_uuid}]}
+        else:
+            data = {'spec_list':[{
+                       'network_uuid': network_uuid,
+                       'requested_ip_address': ip_address,
+                       'is_connected': True,
+                       'request_ip': True,
+                       'model': driver}]}
+        return self.make_request(endpoint, 'post', data=data)
+
+    def detach_nic(self, vm_uuid, nic_mac):
+        logging.info("Detach nic from VM")
+        endpoint = urljoin(self.base_url, "vms/%s/nics/%s" % (vm_uuid, nic_mac))
+        return self.make_request(endpoint, 'delete')
+
+    def get_nics(self, vm_uuid):
+        logging.info("Fetch nics")
+        endpoint = urljoin(self.base_url, "vms/%s/nics/" % (vm_uuid))
+        return self.make_request(endpoint, 'get')
+
+    def attach_disk(self, vm_uuid, device_bus, disk_size, is_cdrom, device_index, **empty_or_clone):
         '''
         Attach disk/cdrom, need to indicate clone_from_img_service or clone_from_adsf_file when is_empty equals False
         '''
-        logging.info("Prism creating and attaching disk, device_bus:{}, disk_size: {}, is_cdrom: {}, is_empty: {}".format(device_bus, disk_size, is_cdrom, is_empty))
+        logging.info("Prism creating and attaching disk, device_bus:{}, disk_size: {}, is_cdrom: {}, empty_or_clone: {}".format(device_bus, disk_size, is_cdrom, empty_or_clone))
         endpoint = urljoin(self.base_url, "vms/%s/disks/attach" % vm_uuid)
         if is_cdrom:
             is_cdrom = "true"
         else:
             is_cdrom = "false"
-        if is_empty[0]:
+        if empty_or_clone['is_empty']:
             data = {"vm_disks": [{"disk_address":{"device_bus":device_bus, "device_index":device_index},
                     "is_cdrom": is_cdrom,
                     "is_empty": "true",
@@ -448,11 +511,11 @@ class PrismApi(PrismSession):
                          "size": disk_size*1024*1024*1024}
                     }]}
         else:
-            if is_empty[1] == 'clone_from_img_service':
+            if empty_or_clone['clone'] == 'clone_from_img_service':
                 if is_cdrom:
                     images = self.list_images()
                     for image in images['entities']:
-                        if self.image_name_kickstart_iso == image['name']:
+                        if image['name'] == 'seed_iso':
                             vmdisk_uuid = image['vm_disk_id']
                 else:
                     logging.info('OPPS.Clone disk form img_service has not yet support.')
@@ -467,6 +530,7 @@ class PrismApi(PrismSession):
                     }}]}
             else:
                 logging.info('OPPS.Clone disk form adsf file path has not yet support.')
+        logging.info('data to attach disk is \n'+str(data))
         return self.make_request(endpoint, 'post', data=data)
 
     def detach_disk(self, vm_uuid, device_bus, vmdisk_uuid, device_index):
@@ -532,7 +596,7 @@ class PrismApi(PrismSession):
                 "spec_list": [
                 {
                     "memory_mb": memory,
-                    "name": vm_name,
+                    "name": self.vm_name+'_'+vm_name,
                     "num_cores_per_vcpu": cores_per_vcpu,
                     "num_vcpus": vcpus,
                     "override_network_config": override_network_config
@@ -561,6 +625,10 @@ class PrismApi(PrismSession):
             % (filter_name, filter_value))
         return self.make_request(endpoint, 'get')
 
+    def get_vm_by_uuid(self, vm_uuid):
+        endpoint = urljoin(self.base_url, "vms/%s" % vm_uuid)
+        return self.make_request(endpoint, 'get')
+
 class NutanixVM(VMResource):
     def __init__(self, params):
         super(NutanixVM, self).__init__(params)
@@ -569,7 +637,9 @@ class NutanixVM(VMResource):
         # VM parameters
         #self.vm_name = params.get('vm_name', '*/VM/*')
         self.vm_name = params['VM']['vm_name']
-
+        self.cpu = params['Flavor']['cpu']
+        self.memory = params['Flavor']['memory']
+        self.image_name = params['VM']['image_name']
         # VM access parameters
         #self.vm_username = params.get('username', '*/VM/*')
         self.vm_username = params['VM']['username']
@@ -579,13 +649,44 @@ class NutanixVM(VMResource):
         self.vm_user_data = params['VM']['custom_data']
         #self.network_uuid = params.get('network_uuid', '*/VM/*')
         self.network_uuid = params['VM']['network_uuid']
+        self.private_network_uuid = params['VM']['private_network_uuid']
+        self.private_network_subnet = params['VM']['private_network_subnet']
         self.ssh_pubkey = utils_lib.get_public_key()
         self.arch = 'x86_64'
-        self.vm_custom_file = None
+        self.vm_custom_file = 'man-page-day.sh'
         self.provider = params['Cloud']['provider']
+        self.subscription_username = params['Subscription']['username']
+        self.subscription_password = params['Subscription']['password']
+        self.subscription_serverurl = params['Subscription'].get('serverurl')
+        self.subscription_baseurl = params['Subscription'].get('baseurl')
+        self.host_username = params['Credential']['host_username']
+        self.host_password = params['Credential']['host_password']
+        self.net_bandwidth = params['VM']['net_bandwidth']
+        self.user_data = None
+        self.vm1_ip = ''
 
         self.prism = PrismApi(params)
 
+    @property
+    def is_secure_boot(self):
+        """
+        vm provisioned with secure boot or not
+        :return: True|False
+        """
+        return self.prism.if_secure_boot
+
+    @property
+    def is_uefi_boot(self):
+        """
+        vm provisioned with uefi or not
+        :return: True|False
+        """
+        return self.prism.if_uefi_boot
+
+    @property
+    def ipv6_address(self):
+        NotImplementedError
+
     @property
     def data(self):
         if not self._data:
@@ -596,6 +697,9 @@ class NutanixVM(VMResource):
                     break
         return self._data
 
+    def refresh_data(self):
+        self._data = None
+
     @property
     def floating_ip(self):
         f_ip = None
@@ -611,6 +715,24 @@ class NutanixVM(VMResource):
             host_uuid.append(host['uuid'])
         return host_uuid
 
+    @property
+    def disk_count(self):
+        logging.info('vm_disk_info in vm self show: \n' + str(self.show()['vm_disk_info']))
+        disk_count = 0
+        for disk in self.show()['vm_disk_info']:
+            if not disk['is_cdrom']:
+                disk_count += 1
+        return disk_count
+
+    @property
+    def host_ip(self):
+        self._data = None
+        for host in self.prism.list_hosts_detail()["entities"]:
+            if host['uuid'] == self.data.get('host_uuid'):
+                ip = host['hypervisor_address']
+                break
+        return ip
+
     def host_cpu_model(self):
         self._data = None
         for host in self.prism.list_hosts_detail()["entities"]:
@@ -625,7 +747,14 @@ class NutanixVM(VMResource):
             if host['uuid'] == self.data.get('host_uuid'):
                 cpu_num = host['num_cpu_sockets']
                 break
-        return cpu_num       
+        return cpu_num
+
+    def host_gpu_info(self):
+        self._data = None
+        for host in self.prism.list_hosts_detail()["entities"]:
+            if host['uuid'] == self.data.get('host_uuid'):
+                gpu_info = host['host_gpus']
+        return gpu_info   
 
     def vm_host_uuid(self):
         self._data = None
@@ -644,20 +773,21 @@ class NutanixVM(VMResource):
                 logging.error("progress status of task is Failed")
                 break
 
-    def create(self, wait=False):
-        logging.info("Create VM")
+    def create(self, single_nic=True, wait=True, vm_name=None):
+        logging.info("Create VM, single_nic is " + str(single_nic))
         self.prism.vm_user_data = self.vm_user_data
+        self.prism.user_data = self.user_data
         self.prism.vm_custom_file = self.vm_custom_file
-        res = self.prism.create_vm(self.ssh_pubkey)
+        res = self.prism.create_vm(self.ssh_pubkey, single_nic, vm_name)
         if wait:
             self.wait_for_status(
                 res['task_uuid'], 60,
                 "Timed out waiting for server to get created.")
         self._data = None
         
-    def create_by_ISO_kickstart(self, wait=False):
+    def create_by_ISO_kickstart(self, wait=False, single_nic=True, vm_name=None):
         logging.info("Create VM by ISO kickstart")
-        res = self.prism.create_vm_ISO_kickstart()
+        res = self.prism.create_vm_ISO_kickstart(single_nic, vm_name)
         logging.debug("res is " + str(res))
         if wait:
             self.wait_for_status(
@@ -665,7 +795,35 @@ class NutanixVM(VMResource):
                 "Timed out waiting for server to get created.")
         self._data = None
 
-    def delete(self, wait=False):
+    def create_vm_by_acli(self, vm_name, memory, cores_per_vcpu, vcpus, if_uefi_boot, if_vtpm):
+        logging.info('Create VM by acli')
+        if self.prism.machine_type == 'q35':
+            cdrom_bus = 'sata'
+        else:
+            cdrom_bus = 'ide'
+        create_vm_cmd1 = 'acli vm.create %s memory=%s num_cores_per_vcpu=%s num_vcpus=%s uefi_boot=%s virtual_tpm=%s machine_type=%s' \
+            %(vm_name, memory, cores_per_vcpu, vcpus, if_uefi_boot, if_vtpm, self.prism.machine_type)
+        create_vm_cmd2 = 'acli vm.disk_create %s clone_from_image=%s' \
+            %(vm_name, self.image_name)
+        create_vm_cmd2_1 = 'acli vm.disk_create %s cdrom=true bus=%s clone_from_adsf_file=/%s/seed.iso' \
+            %(vm_name, cdrom_bus, self.prism.get_container()['name'])
+        create_vm_cmd3 = 'acli vm.nic_create %s connected=true network=%s request_ip=true' \
+            % (vm_name, self.network_uuid)
+        start_vm_cmd = 'acli vm.on %s' % vm_name
+        create_vm = self.cvm_cmd(create_vm_cmd1)
+        if 'Unknown keyword argument' in create_vm:
+            logging.error('There is an unknown keyword argument when create VM: \n' % create_vm)
+        elif not 'complete' in create_vm:
+            logging.error('Create VM not complete: \n %s' % create_vm)
+        for cmd, error_log in zip([create_vm_cmd2, create_vm_cmd2_1, create_vm_cmd3, start_vm_cmd], \
+            ['Attach disk not complete', 'Attach cd-rom not complete', 'Create NIC not complete', 'Start VM not complete']):
+            cmd_res = self.cvm_cmd(cmd)
+            if not 'complete' in cmd_res:
+                logging.error('%s \n %s' %(error_log, cmd_res))
+        vm = self.get_vm_by_filter('vm_name', vm_name)
+        return vm
+
+    def delete(self, wait=True):
         logging.info("Delete VM")
         res = self.prism.delete_vm(self.data.get('uuid'))
         if wait:
@@ -673,7 +831,7 @@ class NutanixVM(VMResource):
                 res['task_uuid'], 30,
                 "Timed out waiting for server to get deleted.")
 
-    def start(self, wait=False):
+    def start(self, wait=True):
         logging.info("start vm")
         res = self.prism.start_vm(self.data.get('uuid'))
         if wait:
@@ -775,6 +933,11 @@ class NutanixVM(VMResource):
         mem_gb = (self.data.get('memory_mb') / 1024)
         return mem_gb
 
+    def get_vgpu_info(self):
+        self._data = None
+        vgpu_info = self.data.get('vm_gpus')
+        return vgpu_info
+
     def update_vcpu_num(self, vcpu_num_target):
         '''
         If target vCPU number is less than current, the key steps will be:
@@ -884,7 +1047,7 @@ class NutanixVM(VMResource):
                         break
                 else:
                     if self.get_cpu_passthrough(enabled=False):
-                        break                    
+                        break
 
     def get_memory_vnuma(self):
         logging.info("Get VM cpu passthrough status.")
@@ -912,11 +1075,53 @@ class NutanixVM(VMResource):
                 if self.get_memory_vnuma() == vnuma_num_target:
                     break
 
-    def attach_disk(self, device_bus, disk_size, is_cdrom, device_index, *is_empty, wait=False):
+    def assign_vgpu(self, device_name):
+        '''
+        Key steps:
+        1. Power off VM
+        2. Assign vgpu
+        3. Power on VM
+        '''
+        if self.is_started():
+            self.stop(wait=True)
+            
+        logging.info("Add vgpu to VM")
+        res = self.cvm_cmd("acli vm.gpu_assign %s gpu=%s" % (self.data.get('uuid'), device_name))
+        logging.info(res)
+        if "pending" in res.lower() and "cannot" not in res.lower():
+            logging.info("vGPU has assigned to VM successfully.")
+            self.start(wait=True)
+            for count in utils_lib.iterate_timeout(
+                    10, "Timed out waiting for verify vGPU assignment."):
+                if self.get_vgpu_info():
+                    break
+
+    def deassign_vgpu(self, device_name):
+        '''
+        Key steps:
+        1. Power off VM
+        2. Deassign vgpu
+        3. Power on VM
+        '''
+        if self.is_started():
+            self.stop(wait=True)
+            
+        logging.info("Remove vgpu from VM")
+        res = self.cvm_cmd("acli vm.gpu_deassign %s gpu=%s" % (self.data.get('uuid'), device_name))
+        logging.info(res)
+        if "pending" in res.lower() and "cannot" not in res.lower():
+            logging.info("vGPU has deassigned from VM successfully.")
+            self.start(wait=True)
+            for count in utils_lib.iterate_timeout(
+                    10, "Timed out waiting for verify vGPU deassignment."):
+                if not self.get_vgpu_info():
+                    break
+
+    def attach_disk(self, device_bus, disk_size, is_cdrom, device_index, wait=False, **empty_or_clone):
         '''
         Attach disk/cdrom, device_info args including disksize, is cdrom or not, and is empty or not
         '''
-        res = self.prism.attach_disk(self.data.get('uuid'), device_bus, disk_size, is_cdrom, device_index, *is_empty)
+        res = self.prism.attach_disk(self.data.get('uuid'), device_bus, disk_size, is_cdrom, device_index, **empty_or_clone)
         time.sleep(30)
 
         if wait:
@@ -924,7 +1129,7 @@ class NutanixVM(VMResource):
                 res['task_uuid'], 30,
                 "Timed out attaching disk.")
 
-    def detach_disk(self, device_bus, vmdisk_uuid, device_index, wait=False):
+    def detach_disk(self, device_bus, vmdisk_uuid, device_index, wait=True):
         '''
         Attach disk according to vmdisk_uuid
         '''
@@ -939,12 +1144,26 @@ class NutanixVM(VMResource):
 
     def detach_block(self, disk, wait=True, force=False):
         raise NotImplementedError
-    
-    def detach_nic(self, nic, wait=True, force=False):
-        raise NotImplementedError
 
-    def attach_nic(self, nic, wait=True, timeout=120):
-        raise NotImplementedError
+    def attach_nic(self, network_uuid=None, ip_address=None, driver='virtio', wait=True, timeout=120):
+        if network_uuid==None:
+            network_uuid = self.network_uuid
+        res = self.prism.attach_nic(self.data.get('uuid'), network_uuid, ip_address, driver)
+        if wait:
+            self.wait_for_status(
+                res['task_uuid'], timeout,
+                "Timed out waiting for attaching NIC.")
+
+    def detach_nic(self, nic_mac, wait=True, force=False):
+        res = self.prism.detach_nic(self.data.get('uuid'), nic_mac)
+        if wait:
+            self.wait_for_status(
+                res['task_uuid'], 60,
+                "Timed out waiting for detaching NIC.")
+
+    def get_nic(self):
+        res = self.prism.get_nics(self.data.get('uuid'))
+        return res
 
     def get_console_log(self):
         raise UnSupportedAction('No such operation in nutanix')
@@ -970,14 +1189,14 @@ class NutanixVM(VMResource):
     def unpause(self, wait=False):
         raise NotImplementedError
 
-    def disk_count(self):
-        raise NotImplementedError
-
     def get_disk_uuid(self, device_type, device_index):
         disk_uuid = None
         for disk in self.show()['vm_disk_info']:
             if disk['disk_address']['device_bus'] == device_type and disk['disk_address']['device_index'] == device_index:
-                disk_uuid = disk['disk_address']['vmdisk_uuid']
+                if 'vmdisk_uuid' in disk['disk_address']:
+                    disk_uuid = disk['disk_address']['vmdisk_uuid']
+                else:
+                    disk_uuid = disk['disk_address']['device_uuid']
         return disk_uuid
 
     def take_snapshot(self, snpst_name, wait=False):
@@ -1002,7 +1221,7 @@ class NutanixVM(VMResource):
                 res['task_uuid'], 60,
                 "Timed out waiting for restoring VM.")
 
-    def clone_vm(self, clone_form_vm_or_snapshot, vm_name, memory, cores_per_vcpu, vcpus, override_network_config, fresh_install, vm_custom_file, vm_userdata_file, wait=False):
+    def clone_vm(self, clone_form_vm_or_snapshot, vm_name, memory, cores_per_vcpu, vcpus, override_network_config, fresh_install, vm_custom_file, vm_userdata_file, wait=True):
         if clone_form_vm_or_snapshot == 'clone_from_vm':
              uuid = self.data.get('uuid')
         else:
@@ -1026,6 +1245,14 @@ class NutanixVM(VMResource):
                     break
         return vm_data
 
+    def list_networks_address(self, network_uuid):
+        address_data = self.prism.list_networks_address(network_uuid)
+        address_list = []
+        for nic in address_data["entities"]:
+            ip=nic["ip_address"]
+            address_list.append(ip)
+        return address_list
+
 class NutanixVolume(StorageResource):
     '''
     Volume class
@@ -1054,6 +1281,7 @@ class NutanixVolume(StorageResource):
         raise NotImplementedError
 
     def modify_disk_size(self, origin_disk_size, device_type, disk_index:('int >= 0'), expand_size:('int > 0'), wait=True):
+        logging.info("Expand disk size with value %s" % expand_size)
         disk_uuid = self.vm.get_disk_uuid(device_type, disk_index)
         res = self.prism.expand_disk(disk_uuid=disk_uuid, disk_size=origin_disk_size+expand_size, device_index=disk_index)
         self.vm.wait_for_status(
