@@ -1,3 +1,4 @@
+from copy import deepcopy
 import os
 import random
 import re
@@ -22,7 +23,6 @@ except ImportError:
     from yaml import Loader, Dumper
 
 LOG = logging.getLogger('os_tests.os_tests_run')
-logging.basicConfig(level=logging.INFO)
 
 def init_args():
     parser = argparse.ArgumentParser(
@@ -30,19 +30,21 @@ def init_args():
     parser.add_argument('-l', dest='is_listcase', action='store_true',
                     help='list supported cases without run', required=False)
     parser.add_argument('-p', dest='pattern', default=None, action='store',
-                    help='filter case by name, add --strict for matching exactly', required=False)
+                    help='filter case by name and run it in specified order, add --strict for matching exactly', required=False)
     parser.add_argument('--strict', dest='is_strict', action='store_true',
                     help='match exactly if -p or -s specified', required=False)
     parser.add_argument('-s', dest='skip_pattern', default=None, action='store',
                     help='skip cases, add --strict for skipping exactly', required=False)
     parser.add_argument('--verifydoc', dest='verifydoc', action='store_true',
                     help='verify or show case doc only', required=False)
+    parser.add_argument('--uploaddoc', dest='uploaddoc', default=None, action='store',
+                    help='upload doc to specific test case management system, eg. polarion', required=False)
     parser.add_argument('--filter_by', dest='filter_by', default='case_name', action='store',
                     help="filter by 'case_name'(default),'case_tag','case_file','component','bugzilla_id',\
                         'is_customer_case','testplan','maintainer','description','key_steps',\
                         'expect_result','debug_want'", required=False)
-    parser.add_argument('--host', dest='remote_node', default=None, action='store',
-                    help='run tests on remote node', required=False)
+    parser.add_argument('--hosts', dest='remote_nodes', default=None, action='store',
+                    help='run tests on remote nodes', required=False)
     parser.add_argument('--port', dest='remote_port', default=22, action='store',
                     help='port for ssh connection, default is 22', required=False)
     parser.add_argument('--user', dest='remote_user', default=None, action='store',
@@ -61,66 +63,53 @@ def init_args():
     return args
 
 def init_provider(params=None):
-    vm = None
-    disk = None
-    nic = None
-    if 'aws' in params['Cloud']['provider']:
+    vms = []
+    disks = []
+    nics = []
+    supported_platforms = ['aws', 'openstack', 'ali', 'nutanix', 'google', 'libvirt', 'openshift']
+    provider = params['Cloud']['provider']
+    if not provider:
+        LOG.info("no provider found in profile".format(provider, supported_platforms))
+        sys.exit(1)
+    elif provider not in supported_platforms:
+        LOG.info("{} not in supported platforms:{}".format(provider, supported_platforms))
+        sys.exit(1)
+
+    if 'aws' in provider:
         from .resources_aws import EC2VM,EC2Volume,EC2NIC
-        vm = EC2VM(params)
-        vm.create()
-        # init disk and do not create disk at very beginning
-        disk = EC2Volume(params)
-        nic = EC2NIC(params)
-    if 'openstack' in params['Cloud']['provider']:
+        # init resources only without create them at very beginning
+        vms.extend([EC2VM(params),EC2VM(params)])
+        disks.append(EC2Volume(params))
+        nics.append(EC2NIC(params))
+    if 'openstack' in provider:
         from .resources_openstack import OpenstackVM
-        vm = OpenstackVM(params)
-        vm.create()
-        disk = None
-    if 'ali' in params['Cloud']['provider']:
+        vms.append(OpenstackVM(params))
+    if 'ali' in provider:
         from .resources_alicloud import AlibabaVM
-        vm = AlibabaVM(params)
-        vm.create(wait=True)
-        if vm.is_stopped():
-            vm.start(wait=True)
-        disk = None
-    if 'nutanix' in params['Cloud']['provider']:
+        vms.append(AlibabaVM(params))
+    if 'nutanix' in provider:
         from .resources_nutanix import NutanixVM,NutanixVolume
-        vm = NutanixVM(params)
-        vm.create(wait=True)
-        if vm.is_stopped():
-            vm.start(wait=True)
-        disk = NutanixVolume(params)
-    if 'google' in params['Cloud']['provider']:
+        vms.append(NutanixVM(params))
+        disks.append(NutanixVolume(params))
+    if 'google' in provider:
         from .resources_gcp import GCPVM
-        vm = GCPVM(params)
-        vm.create(wait=True)
-        if vm.is_stopped():
-            vm.start(wait=True)
-        disk = None
-    if 'libvirt' in params['Cloud']['provider']:
+        vms.append(GCPVM(params))
+    if 'libvirt' in provider:
         from .resources_libvirt import LibvirtVM
-        vm = LibvirtVM(params)
-        vm.create(wait=True)
-        if vm.is_stopped():
-            vm.start(wait=True)
-        disk = None
-    if 'openshift' in params['Cloud']['provider']:
+        vms.append(LibvirtVM(params))
+    if 'openshift' in provider:
         from .resources_openshift import OpenShiftVM
-        vm = OpenShiftVM(params)
-        vm.create(wait=True)
-        if vm.is_stopped():
-            vm.start(wait=True)
-        disk = None
+        vms.append(OpenShiftVM(params))
 
-    return vm, disk, nic
+    return vms, disks, nics
 
-def init_ssh(params=None, timeout=600, interval=10, log=None):
+def init_ssh(params=None, timeout=600, interval=10, log=None, rmt_node=None):
     if log is None:
         LOG_FORMAT = '%(levelname)s:%(message)s'
         log = logging.getLogger(__name__)
         logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
     ssh = rmt_ssh.RemoteSSH()
-    ssh.rmt_node = params['remote_node']
+    ssh.rmt_node = rmt_node or params['remote_node']
     ssh.port = params['remote_port']
     ssh.rmt_user = params['remote_user']
     ssh.rmt_password = params['remote_password']
@@ -129,43 +118,98 @@ def init_ssh(params=None, timeout=600, interval=10, log=None):
     ssh.timeout = timeout
     ssh.interval = interval
     ssh.create_connection()
-    if ssh.ssh_client is not None:
-        return ssh
-    return None
+    return ssh
 
-def init_connection(test_instance, timeout=600, interval=10):
-    if test_instance.params['remote_node'] is None:
+def init_connection(test_instance, timeout=600, interval=10, rmt_node=None, vm=None):
+    if not test_instance.params['remote_node'] and not rmt_node and not vm and not test_instance.vm:
         return
-    test_instance.log.info("remote_node specified, all tests will be run in {}".format(test_instance.params['remote_node']))
+    new_vm_ip = None
+    is_master_vm = False
+    if test_instance.vm:
+        new_vm_ip =  test_instance.vm.floating_ip
+        if new_vm_ip != test_instance.params['remote_node']:
+            test_instance.params['remote_node'] = new_vm_ip
+            if test_instance.params['remote_nodes']:
+                test_instance.params['remote_nodes'][0] = new_vm_ip
+            else:
+                test_instance.params['remote_nodes'].append(new_vm_ip)
+            test_instance.log.info("set default remote_node to new address {}".format(test_instance.params['remote_node']))
+            is_master_vm = True
+    rmt_node = rmt_node or test_instance.params['remote_node'] or None
+    if vm:
+        if hasattr(vm, 'floating_ip'):
+            rmt_node = vm.floating_ip
+        if vm.dead_count > 4:
+            test_instance.fail("cannot connect to vm over 4 times, skip retry to connect it")
+    test_instance.log.info("remote_node found, init connection {}".format(rmt_node))
+    ssh_exists = False
+    is_active = False
     try:
-        if hasattr(test_instance.SSH, 'ssh_client'):
-            test_instance.SSH.log = test_instance.log
-            ret, _, _ = test_instance.SSH.cli_run(cmd='uname -r')
-            if ret == 0:
-                test_instance.log.info("connection is live")
-                return
+        ssh_num = 0
+        for i, ssh in enumerate(test_instance.SSHs):
+            ssh.log = test_instance.log
+            if ssh.rmt_node == new_vm_ip:
+                test_instance.SSH = ssh
+            if ssh.rmt_node == rmt_node:
+                ssh_exists = True
+                ssh_num = i
+                ret, _, _ = ssh.cli_run(cmd='uname -r')
+                if ret == 0:
+                    test_instance.log.info("connection is live")
+                    is_active = True
+                break
+        for tmp_ssh in test_instance.SSHs:
+            if tmp_ssh.rmt_node == test_instance.params['remote_node']:
+                test_instance.SSH = tmp_ssh
+                break
+        if is_active:
+            return
     except AttributeError:
         pass
     except Exception:
         test_instance.log.info("connection is not live")
-    test_instance.SSH = init_ssh(params=test_instance.params, timeout=timeout, interval=interval, log=test_instance.log)
-    if test_instance.SSH is None:
-        if test_instance.vm:
+    if ssh_exists:
+        test_instance.log.info("found exists dead connection, re-connect")
+        test_instance.SSHs[ssh_num].timeout = timeout
+        test_instance.SSHs[ssh_num].create_connection()
+        ssh = test_instance.SSHs[ssh_num]
+    else:
+        ssh = init_ssh(params=test_instance.params, timeout=timeout, interval=interval, log=test_instance.log, rmt_node=rmt_node)
+        if ssh.ssh_client:
+            test_instance.SSHs.append(ssh)
+    if not ssh.ssh_client:
+        vm = vm or test_instance.vm
+        if vm:
             try:
-                test_instance.vm.get_console_log()
+                vm.get_console_log()
             except NotImplementedError:
-                test_instance.log.info("{} not implement this func: get_console_log".format(test_instance.vm.provider))
+                test_instance.log.info("{} not implement this func: get_console_log".format(vm.provider))
+            vm.dead_count += 1
+            test_instance.log.info("vm dead times:{}".format(vm.dead_count))
+            if vm.dead_count == 2:
+                test_instance.log.info("vm cannot connect in {} times, restart it".format(vm.dead_count))
+                vm.stop()
+                vm.start()
+        test_instance.fail("Cannot make ssh connection to remote, please check")
+    else:
+        vm = vm or test_instance.vm
+        if vm:
+            vm.dead_count = 0
+    for tmp_ssh in test_instance.SSHs:
+        if tmp_ssh.rmt_node == test_instance.params['remote_node']:
+            test_instance.SSH = tmp_ssh
+            break
 
-        test_instance.skipTest("Cannot make ssh connection to remote, please check")
-
-def send_ssh_cmd(rmt_node, rmt_user, rmt_password, command):
+def send_ssh_cmd(rmt_node, rmt_user, rmt_password, command, timeout=60):
     ssh = rmt_ssh.RemoteSSH()
     ssh.rmt_node = rmt_node
     ssh.rmt_user = rmt_user
     ssh.rmt_password = rmt_password
     ssh.create_connection()
-    status, outputs = ssh.remote_excute(command)
+    status, outputs = ssh.remote_excute(command, timeout)
+    logging.info('\n command: %s \n status %s \n outputs %s \n' % (command, status, outputs))
     ssh.close()
+
     return [status,outputs]
 
 def get_cfg(cfg_file = None):
@@ -177,6 +221,9 @@ def get_cfg(cfg_file = None):
         sys.exit(1)
         return cfg_file, None
     keys_data = load_yaml(yaml_file=cfg_file)
+    if keys_data.get('remote_nodes'):
+        keys_data['remote_nodes'] = keys_data['remote_nodes'].split(',')
+        keys_data['remote_node'] =  keys_data['remote_nodes'][0]
     return cfg_file, keys_data
 
 def load_yaml(yaml_file = None, yaml_content = None):
@@ -201,10 +248,15 @@ def init_case(test_instance):
         cfg_file, keys_data = get_cfg()
         test_instance.params = keys_data
     results_dir = test_instance.params['results_dir']
-    debug_dir = results_dir + "/debug"
+    attachment_dir = results_dir + "/attachments"
     test_instance.log_dir = results_dir
     if not os.path.exists(results_dir):
         os.mkdir(results_dir)
+    if not os.path.exists(attachment_dir):
+        os.mkdir(attachment_dir)
+    test_class_name = test_instance.__class__.__name__
+    case_dir = '.'.join([test_class_name, test_instance.id()])
+    debug_dir = os.path.join(attachment_dir, case_dir)
     if not os.path.exists(debug_dir):
         os.mkdir(debug_dir)
     case_log = test_instance.id() + ".debug"
@@ -215,7 +267,7 @@ def init_case(test_instance):
     for handler in logging.root.handlers[:]:
         handler.close()
         logging.root.removeHandler(handler)
-    FORMAT = "%(levelname)s:%(message)s"
+    FORMAT = '%(asctime)s:%(levelname)s:%(message)s'
     logging.basicConfig(level=logging.INFO, format=FORMAT, filename=log_file)
     test_instance.log.info("-"*80)
     test_instance.log.info("Code Repo: {}".format(test_instance.params['code_repo']))
@@ -229,12 +281,29 @@ def init_case(test_instance):
         else:
             test_instance.log.info("key:{}, val:{}".format(key, test_instance.params[key]))
     test_instance.log.info("-"*80)
-    if test_instance.params['remote_node'] is not None:
-        init_connection(test_instance)
-        test_instance.SSH.log = test_instance.log
-        if  test_instance.SSH.ssh_client is None:
-            test_instance.skipTest("Cannot make ssh connection to remote, please check")
-    node_info = "{}/node_info".format(debug_dir)
+    test_instance.ssh_timeout = 180
+    test_instance.default_boot_index = None
+    if test_instance.vm:
+        if test_instance.vm.dead_count > 4:
+            test_instance.fail("cannot connect to vm over 4 times, skip retry")
+        if test_instance.vm.is_metal:
+            test_instance.ssh_timeout = 1200
+        if not test_instance.vm.exists():
+            test_instance.vm.create()
+        if test_instance.vm.is_stopped():
+            test_instance.vm.start(wait=True)
+        test_instance.params['remote_port'] = test_instance.vm.port or 22
+
+    if test_instance.is_rmt:
+        test_instance.log.info('ssh connection timeout:{}'.format(test_instance.ssh_timeout))
+        init_connection(test_instance, timeout=test_instance.ssh_timeout)
+        if not test_instance.params['remote_node']:
+            test_instance.fail("remote_node not found")
+        if test_instance.SSH:
+            test_instance.SSH.log = test_instance.log
+            if  not test_instance.SSH.ssh_client:
+                test_instance.fail("Cannot make ssh connection to remote, please check!")
+    node_info = "{}/node_info".format(attachment_dir)
     node_info_data = {}
     if not os.path.exists(node_info):
         test_instance.log.info("retrive node info.")
@@ -258,6 +327,30 @@ def finish_case(test_instance):
     """
     pass
 
+def case_to_polarion(case=None,cfg=None):
+    from os_tests.libs.polarion import PolarionCase
+    polarion_case = PolarionCase(cfg=cfg)
+    LOG.info("upload {}".format(case.id()))
+    yaml_data = {}
+    try:
+        yaml_data = load(case._testMethodDoc, Loader=Loader)
+        if not hasattr(yaml_data,'get'):
+            yaml_data = {}
+            yaml_data['case_name'] = case.id()
+        else:
+            yaml_data['case_name'] = case.id()
+    except Exception as err:
+        yaml_fail = err
+        yaml_data['case_name'] = case.id()
+        LOG.info("please verify its docstring before uploading")
+        return False
+    polarion_case.casedoc = yaml_data
+    if polarion_case.query_case():
+        polarion_case.update_case()
+    else:
+        polarion_case.add_new()
+    return True
+
 def filter_case_doc(case=None, patterns=None, skip_patterns=None, filter_field='case_name', strict=False, verify_doc=False ):
     if patterns is None and skip_patterns is None and not verify_doc:
         return True
@@ -323,9 +416,10 @@ def filter_case_doc(case=None, patterns=None, skip_patterns=None, filter_field='
                     is_skip = True
                     break
     if verify_doc and is_select and not is_skip:
-        expect_fields = ['case_tag','case_name','case_file','component','bugzilla_id',
-                        'is_customer_case','testplan','maintainer','description','key_steps',
-                        'expect_result','debug_want']
+        expect_fields = ['case_tags', 'title', 'importance', 'subsystem_team', 'automation_drop_down', 
+                        'linked_work_items', 'automation_field', 'setup_teardown', 'environment', 'case_name', 
+                        'component', 'bug_id', 'is_customer_case', 'testplan', 'maintainer', 'description', 
+                        'key_steps', 'expected_result', 'debug_want']
         print("="*20)
         print(case.id())
         print(case._testMethodDoc)
@@ -334,25 +428,26 @@ def filter_case_doc(case=None, patterns=None, skip_patterns=None, filter_field='
             print(yaml_fail)
             return is_select and not is_skip
         for i in expect_fields:
-            if not yaml_data.get(i):
+            if yaml_data.get(i) == None:
                 print('missing {}'.format(i))
     return is_select and not is_skip
 
-def msg_to_syslog(test_instance, msg=None):
+def msg_to_syslog(test_instance, cmd='sudo virt-what', msg=None):
     '''
     Save msg to journal log and dmesg.
     Arguments:
         test_instance {Test instance} -- unittest.TestCase instance
+        cmd {string} -- append cmd output to journal and dmesg
         msg {string} -- msg want to save, default is casename
-    Return:
-        arm: return True
-        other: return False
     '''
     if msg is None:
         msg = test_instance.id()
-    cmd = "sudo echo os-tests:{} | systemd-cat -p info".format(msg)
+    output = ''
+    if cmd:
+        output = run_cmd(test_instance, cmd)
+    cmd = 'sudo echo os-tests:"{} {}" | systemd-cat -p info'.format(msg, output)
     run_cmd(test_instance, cmd, expect_ret=0)
-    cmd = "sudo bash -c 'echo \"{}\" > /dev/kmsg'".format(msg)
+    cmd = "sudo bash -c 'echo \"{} {}\" > /dev/kmsg'".format(msg,output)
     run_cmd(test_instance, cmd, expect_ret=0)
 
 def run_cmd(test_instance,
@@ -367,13 +462,15 @@ def run_cmd(test_instance,
             cancel_not_kw=None,
             cancel_ret=None,
             cancel_not_ret=None,
-            timeout=60,
+            timeout=120,
             ret_status=False,
             is_log_output=True,
             cursor=None,
             rmt_redirect_stdout=False,
             rmt_redirect_stderr=False,
-            rmt_get_pty=False
+            rmt_get_pty=False,
+            rmt_node = None,
+            vm = None
             ):
     """run cmd with/without check return status/keywords and save log
 
@@ -401,21 +498,36 @@ def run_cmd(test_instance,
         cursor {string} -- skip content before cursor(line)
         rmt_redirect_stdout {bool} -- ssh command not exit some times, redirect stdout to tmpfile if needed
         rmt_redirect_stderr {bool} -- ssh command not exit some times, redirect stderr to tmpfile if needed
+        rmt_node {string} -- run command on specific rmt node
+        vm {vm} -- run command on specific vm
 
     Keyword Arguments:
         check_ret {bool} -- [whether check return] (default: {False})
     """
     if msg is not None:
         test_instance.log.info(msg)
-    test_instance.log.info("CMD: {}".format(cmd))
     status = None
     output = None
     exception_hit = False
 
     try:
-        if test_instance.params['remote_node'] is not None:
-            status, output = test_instance.SSH.remote_excute(cmd, timeout, redirect_stdout=rmt_redirect_stdout, redirect_stderr=rmt_redirect_stderr,rmt_get_pty=rmt_get_pty)
+        if test_instance.is_rmt:
+            if not test_instance.params['remote_node'] and not rmt_node and not vm:
+                return
+            rmt_node = rmt_node or test_instance.params['remote_node'] or None
+            if vm:
+                if hasattr(vm, 'floating_ip'):
+                    rmt_node = vm.floating_ip
+            SSH = None
+            for i, ssh in enumerate(test_instance.SSHs):
+                ssh.log = test_instance.log
+                if ssh.rmt_node == rmt_node:
+                    SSH = ssh
+                    break
+            test_instance.log.info("CMD: {} on {}".format(cmd, rmt_node))
+            status, output = SSH.remote_excute(cmd, timeout, redirect_stdout=rmt_redirect_stdout, redirect_stderr=rmt_redirect_stderr,rmt_get_pty=rmt_get_pty)
         else:
+            test_instance.log.info("CMD: {}".format(cmd))
             ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, timeout=timeout, encoding='utf-8')
             #ret = subprocess.run(cmd, shell=True, capture_output=True, timeout=timeout, encoding='utf-8')
             status = ret.returncode
@@ -433,9 +545,9 @@ def run_cmd(test_instance,
         test_cmd = 'uname -a'
         test_instance.log.info("Test system is alive via cmd:{}. If still fail, check no hang or panic happens.".format(test_cmd))
         try:
-            if test_instance.params['remote_node'] is not None:
-                status, output = test_instance.SSH.remote_excute(test_cmd, timeout)
-                status, output = test_instance.SSH.remote_excute(cmd, timeout, redirect_stdout=rmt_redirect_stdout, redirect_stderr=rmt_redirect_stderr,rmt_get_pty=rmt_get_pty)
+            if test_instance.is_rmt:
+                status, output = SSH.remote_excute(test_cmd, timeout)
+                status, output = SSH.remote_excute(cmd, timeout, redirect_stdout=rmt_redirect_stdout, redirect_stderr=rmt_redirect_stderr,rmt_get_pty=rmt_get_pty)
             else:
                 ret = subprocess.run(test_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, timeout=timeout, encoding='utf-8')
                 status = ret.returncode
@@ -451,14 +563,14 @@ def run_cmd(test_instance,
                    output = ret.stdout
         except Exception as err:
             test_instance.log.error("Run cmd failed again {}".format(err))
-    if status is None and test_instance.vm:
-        try:
-            test_instance.vm.get_console_log()
-        except NotImplementedError:
-            test_instance.log.info("{} not implement this func: get_console_log".format(test_instance.vm.provider))
-        test_instance.vm.stop()
-        test_instance.vm.start()
-        test_instance.params['remote_node'] = test_instance.vm.floating_ip
+    if status is None and test_instance.vms:
+        for vm in test_instance.vms:
+            if vm.exists() and rmt_node == vm.floating_ip:
+                try:
+                    test_instance.vm.get_console_log()
+                except NotImplementedError:
+                    test_instance.log.info("{} not implement this func: get_console_log".format(test_instance.vm.provider))
+
     if cursor is not None and output is not None and cursor in output:
         output = output[output.index(cursor):]
     if is_log_output:
@@ -479,7 +591,7 @@ def run_cmd(test_instance,
     if expect_kw is not None:
         for key_word in expect_kw.split(','):
             if output.count('\n') > 5:
-                find_list = re.findall('\n.*{}.*\n'.format(key_word), output)
+                find_list = re.findall('.*{}.*'.format(key_word), output)
             else:
                 find_list = re.findall('.*{}.*'.format(key_word), output)
             if len(find_list) > 0:
@@ -594,6 +706,28 @@ def getboottime(test_instance):
         "Boot time is {}(s)".format(boot_time_sec))
     return boot_time_sec
 
+def is_sev_enabled(test_instance):
+    '''
+    Check whether SEV is enabled on the system.
+    Arguments:
+        test_instance {Test instance} -- unittest.TestCase instance
+    Return:
+        enabled: return True
+        N/A: return False
+    '''
+    if test_instance.vm.provider == 'google':
+        audience = 'aud'
+        cmd = "curl -s -H 'Metadata-Flavor: Google' \
+            'http://metadata/computeMetadata/v1/instance/service-accounts/default/identity?audience={}&format=full'".format(audience)
+        token = run_cmd(test_instance, cmd, expect_ret=0)
+        if test_instance.vm.is_sev_enabled(token, audience):
+            return True
+        else:
+            return False
+    else:
+        raise NotImplementedError
+    return False
+
 def is_arch(test_instance, arch="", action=None):
     '''
     Check whether system is specific system.
@@ -716,6 +850,26 @@ def is_ahv(test_instance, action=None):
         test_instance.log.info("Not a Nutanix AHV system.")
     return False
 
+def is_gcp(test_instance, action=None):
+    '''
+    Check whether system is a gcp system.
+    Arguments:
+        test_instance {Test instance} -- unittest.TestCase instance
+        action {string} -- cancel case if it is not a aws system
+    Return:
+        aws: return True
+        other: return False
+    '''
+    output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/bios_*", expect_ret=0)
+    if 'google' in output.lower():
+        test_instance.log.info("gcp system.")
+        return True
+    else:
+        if action == "cancel":
+            test_instance.skipTest("Cancel it in non gcp system.")
+        test_instance.log.info("Not an gcp system.")
+    return False
+
 def is_metal(test_instance, action=None):
     '''
     Check whether system is a baremetal system.
@@ -726,6 +880,11 @@ def is_metal(test_instance, action=None):
         metal: return True
         other: return False
     '''
+    if test_instance.vm:
+        if action == "cancel":
+            if not test_instance.vm.is_metal:
+                test_instance.skipTest("Cancel it in non metal system.")
+        return test_instance.vm.is_metal
     output_lscpu = run_cmd(test_instance, "lscpu", expect_ret=0)
     if "x86_64" in output_lscpu and "Hypervisor" not in output_lscpu:
         test_instance.log.info("It is a bare metal instance.")
@@ -821,7 +980,7 @@ def pkg_install(test_instance, pkg_name=None, pkg_url=None, force=False):
             pkg_name {string} -- pkg name
             pkg_url {string} -- pkg url or location if it is not in default repo
         """
-        if not is_pkg_installed(test_instance, pkg_name=pkg_name):
+        if not is_pkg_installed(test_instance, pkg_name=pkg_name, cancel_case=False, is_install=False):
             test_instance.log.info("Try install {} automatically!".format(pkg_name))
             if pkg_url is not None:
                 test_instance.log.info("Install {} from {}".format(pkg_name, pkg_url))
@@ -840,7 +999,7 @@ def pkg_install(test_instance, pkg_name=None, pkg_url=None, force=False):
                 cmd = 'sudo yum -y reinstall %s' % pkg_name
             run_cmd(test_instance, cmd, timeout=1200)
 
-        if not is_pkg_installed(test_instance, pkg_name=pkg_name) and pkg_url is not None and force:
+        if not is_pkg_installed(test_instance, pkg_name=pkg_name, cancel_case=False, is_install=False) and pkg_url is not None and force:
             test_instance.log.info('Install without dependences!')
             cmd = 'sudo rpm -ivh %s --nodeps' % pkg_url
             if force:
@@ -939,11 +1098,11 @@ def check_log(test_instance, log_keyword, log_cmd="journalctl -b 0", match_word_
                       rmt_get_pty=rmt_get_pty)
 
     for keyword in log_keyword.split(','):
-        ret = find_word(test_instance, out, keyword, baseline_dict=baseline_dict, skip_words=skip_words)
+        ret, msg = find_word(test_instance, out, keyword, baseline_dict=baseline_dict, skip_words=skip_words)
         if not ret and baseline_dict is not None:
-            test_instance.fail("New {} in {} log".format(keyword, check_cmd))
+            test_instance.fail("New {} in {} log\n{}".format(keyword, check_cmd, '\n'.join(msg)))
         elif not ret:
-            test_instance.fail("Found {} in {} log!".format(keyword, check_cmd))
+            test_instance.fail("Found {} in {} log!\n{}".format(keyword, check_cmd, '\n'.join(msg)))
         else:
             test_instance.log.info("No unexpected {} in {} log!".format(keyword, check_cmd))
 
@@ -998,10 +1157,10 @@ def find_word(test_instance, check_str, log_keyword, baseline_dict=None, skip_wo
     Returns:
         [Bool] -- [True|False]
     """
-    tmp_list = re.findall('.*%s.*\n' % log_keyword, check_str, flags=re.I)
+    tmp_list = re.findall('.*%s.*' % log_keyword, check_str, flags=re.I)
     if len(tmp_list) == 0:
         test_instance.log.info("No %s found!", log_keyword)
-        return True
+        return True, []
     else:
         test_instance.log.info("%s found!", log_keyword)
     if skip_words is not None:
@@ -1009,23 +1168,25 @@ def find_word(test_instance, check_str, log_keyword, baseline_dict=None, skip_wo
             tmp_list = [x for x in tmp_list if skip_word not in x]
     if len(tmp_list) == 0:
         test_instance.log.info("No {} found after skipped {}!".format(log_keyword, skip_words))
-        return True
+        return True, []
+    unknow_log = deepcopy(tmp_list)
     # compare 2 string, if similary over fail_rate, consider it as same.
     fail_rate = 70
     no_fail = True
     check_done = False
     for line1 in tmp_list:
+        # this round go through with regex
         find_it = False
-        if baseline_dict is not None:
+        if baseline_dict:
             for basekey in baseline_dict:
                 for sub_basekey_content in baseline_dict[basekey]["content"].split(';'):
                     check_done = False
-                    if re.search(sub_basekey_content, line1):
+                    if sub_basekey_content and re.search(sub_basekey_content, line1):
                         if baseline_dict[basekey]["status"] == 'active':
-                            test_instance.log.info("Found a similar issue matched in baseline.")
+                            test_instance.log.info("Found a similar log matched in baseline.")
                             find_it = True
                         else:
-                            test_instance.log.info("Found a similar issue matched in baseline. But it is not active, please check manually")
+                            test_instance.log.info("Found a similar log matched in baseline. But it is not active, please check manually")
                             find_it = False
                             no_fail = False
                             check_done = True
@@ -1047,8 +1208,17 @@ def find_word(test_instance, check_str, log_keyword, baseline_dict=None, skip_wo
                               baseline_dict[basekey]["path"]))
                         check_done = True
                         break
-                if find_it or check_done:
+                if find_it:
+                    unknow_log.remove(line1)
                     break
+                if check_done:
+                    break
+    tmp_list = deepcopy(unknow_log)
+    for line1 in tmp_list:
+        # this round go through with content compare
+        find_it = False
+        if baseline_dict:
+            for basekey in baseline_dict:
                 line1_tmp = line1
                 line2_tmp = baseline_dict[basekey]["content"]
                 line1_tmp, line2_tmp = clean_sentence(test_instance, line1_tmp, line2_tmp)
@@ -1082,12 +1252,13 @@ def find_word(test_instance, check_str, log_keyword, baseline_dict=None, skip_wo
                         find_it = False
                         no_fail = False
                     break
-        if not find_it:
-            test_instance.log.info("This is a new exception!")
-            test_instance.log.info("{}".format(line1))
-            no_fail = False
+            if find_it:
+                unknow_log.remove(line1)
+    if unknow_log:
+        test_instance.log.info("Below items are unknow!\n{}".format(unknow_log))
+        no_fail = False
 
-    return no_fail
+    return no_fail, unknow_log
 
 def get_product_id(test_instance):
     cmd = "source /etc/os-release ;echo $VERSION_ID"
@@ -1220,3 +1391,62 @@ def get_public_key(client_user=None):
     public_key.close()
     
     return public_key_str
+
+def normalize_data_size(value_str, order_magnitude="M", factor="1024"):
+    """
+    Normalize a data size in one order of magnitude to another (MB to GB,
+    for example).
+
+    :param value_str: a string include the data default unit is 'B'
+    :param order_magnitude: the magnitude order of result
+    :param factor: the factor between two relative order of magnitude.
+                   Normally could be 1024 or 1000
+    """
+    def __get_unit_index(M):
+        try:
+            return ['B', 'K', 'M', 'G', 'T'].index(M.upper())
+        except ValueError:
+            pass
+        return 0
+
+    regex = r"(\d+\.?\d*)\s*(\w?)"
+    match = re.search(regex, value_str)
+    try:
+        value = match.group(1)
+        unit = match.group(2)
+        if not unit:
+            unit = 'B'
+    except TypeError:
+        raise ValueError("Invalid data size format 'value_str=%s'" % value_str)
+    from_index = __get_unit_index(unit)
+    to_index = __get_unit_index(order_magnitude)
+    scale = int(factor)**(to_index - from_index)
+    data_size = float(value) / scale
+    # Control precision to avoid scientific notaion
+    if data_size.is_integer():
+        return "%.1f" % data_size
+    else:
+        return ("%.20f" % data_size).rstrip('0')
+
+def check_attribute(target, attributes, test_instance=None, cancel_case=True):
+    '''
+    check if has required attribute
+    if test_instance passed, can skip case as required
+    '''
+    if test_instance:
+        func_write = test_instance.log.info
+    else:
+        func_write = print
+    if not target or not attributes:
+        msg = 'target or attributes is None'
+        func_write(msg)
+        return False
+    for attrname in attributes.split(','):
+        if not hasattr(target, attrname):
+            msg = 'no {} found {}'.format(attrname,target)
+            if test_instance and cancel_case:
+                test_instance.skipTest(msg)
+            else:
+                func_write(msg)
+            return False
+    return True
\ No newline at end of file
