@@ -2,6 +2,7 @@ import unittest
 from os_tests.libs import utils_lib
 import json
 import time
+import os
 
 class TestAzureImage(unittest.TestCase):
     def setUp(self):
@@ -15,23 +16,116 @@ class TestAzureImage(unittest.TestCase):
         return int(product_id.split('.')[0])
 
     @property
-    def region(self):
+    def metadata(self):
         cmd = 'curl -s -H Metadata:true --noproxy "*" "http://169.254.169.254/metadata/instance?api-version=2021-02-01"'
-        metadata = utils_lib.run_cmd(self, cmd)
-        return json.loads(metadata)['compute']['location']
+        return json.loads(utils_lib.run_cmd(self, cmd))
 
+
+    # @property
+    # def region(self):
+    #     return self.metadata['compute']['location']
+
+    # @property
+    # def is_gov(self):
+    #     '''
+    #     If it is gov cloud, return true
+    #     '''
+    #     environment = self.metadata['compute']['azEnvironment']
+    #     return environment == "AzureUSGovernmentCloud"
+
+    def _get_generation(self):
+        '''
+        Get genenration from dmesg. Return gen1/gen2.
+        '''
+        ret = utils_lib.run_cmd(self, 'dmesg|grep -w EFI', ret_status=True)
+        if ret == 0:
+            return 'gen2'
+        else:
+            return 'gen1'
+
+    def _check_log(self, filename, ignore_list=[], keyword='err|fail|warn|trace'):
+        cmd = "sudo grep -iE 'err|fail|warn|trace' {}|grep -vE '{}'".format(filename, '|'.join(ignore_list))
+        output = utils_lib.run_cmd(self, cmd, msg="Check {} in {}".format(keyword, filename))
+        if output:
+            self.fail("Unexpected logs exist in {}.\n{}".format(filename, output))
+        else:
+            self.log.info("No unexpected logs in "+filename)
+
+    def _check_file_content(self, basefile=None, testfile=None, expected=None, msg=None, project=None):
+        '''
+        Function to check file content.
+        '''
+        # Print testfile content for recording
+        utils_lib.run_cmd(self, "cat "+testfile)
+        # If base is a file
+        if basefile:
+            src_file = self.data_dir + '/azure/{}'.format(basefile)
+            # Base file content is different in multiple RHEL versions
+            if project:
+                src_file += "_rhel{}".format(project)
+                # If >= RHEL-8 and no specific file, use RHEL-8 file
+                if not os.path.exists(src_file) and project >= 8:
+                    self.log.info("No {}. Use _rhel8 file instead.".format(src_file))
+                    src_file = src_file.replace("_rhel{}".format(project), "_rhel8")
+            # Get the base file path
+            # If remote node, copy the basefile to the remote node /tmp/
+            if self.params['remote_node'] is not None:
+                base_file = "/tmp/"+basefile
+                self.log.info('Copy {} to remote'.format(basefile))
+                self.SSH.put_file(local_file=src_file, rmt_file=base_file)
+            else:
+                base_file = src_file
+            test_file = testfile
+        # If base is a block of strings
+        elif expected:
+            base_file = '/tmp/basefile'
+            utils_lib.run_cmd(self, "echo '''{0}''' > {1};sort {1}>{2}".format(expected, '/tmp/tmpfile', base_file))
+            test_file = '/tmp/testfile'
+            cmd = "sudo cat {}|grep -v '^#'|sed '/^$/d'|sort > {}".format(testfile, test_file)
+            utils_lib.run_cmd(self, cmd)
+        else:
+            self.error('Must specify basefile or expected!')
+        if not msg:
+            msg = "Check "+testfile
+        # Compare files ignore space, empty line and order
+        output = utils_lib.run_cmd(self, "sudo diff -wB {} {}".format(base_file, test_file), msg=msg)
+        self.assertEqual(output, '', "{} and {} are not the same.\n{}".format(base_file, test_file, output))
+
+    def _check_file_not_changed(self, package):
+        utils_lib.run_cmd(self, "sudo rpm -V "+package, expect_ret=0, msg="Verify no file changed in package "+package)
+
+    ######## Test cases ########
     def test_check_bash_history(self):
         '''
+        Verify no old .bash_history exists
         '''
         for user in ['azureuser', 'root']:
             cmd = 'sudo cat ~{}/.bash_history'.format(user)
             utils_lib.run_cmd(self, cmd, expect_not_ret='0', msg='check bash history does not exist in fresh image')
 
     def test_check_cloudinit_cfg_mounts_growpart(self):
-        '''
-        bz: 966888
-        des: make sure there is mounts/growpart in cloud_init_modules group in "/etc/cloud/cloud.cfg"
-        '''
+        """
+        case_tag:
+            cloudinit
+        case_name:
+            test_check_cloudinit_cfg_mounts_growpart
+        component:
+            cloud-init
+        bugzilla_id:
+            966888
+        is_customer_case:
+            False
+        maintainer:
+            xiliang@redhat.com
+        description:
+            make sure there is mounts/growpart in cloud_init_modules group in "/etc/cloud/cloud.cfg"
+        key_steps:
+            1.check cloud config file
+        expect_result:
+            there's 'growpart' or 'mounts' in cloud config file
+        debug_want:
+            cloud.cfg
+        """
         cmd = 'sudo cat /etc/cloud/cloud.cfg'
         utils_lib.run_cmd(self,
                 cmd,
@@ -40,11 +134,28 @@ class TestAzureImage(unittest.TestCase):
                 msg='check /etc/cloud/cloud.cfg to make sure there is mounts/growpart in cloud_init_modules(bz966888)')
 
     def test_check_cloudinit_cfg_no_wheel(self):
-        '''
-        bz: 1549638
-        cm: 01965459
-        des: make sure there is no wheel in default_user's group in "/etc/cloud/cloud.cfg"
-        '''
+        """
+        case_tag:
+            cloudinit
+        case_name:
+            test_check_cloudinit_cfg_no_wheel
+        component:
+            cloud-init
+        bugzilla_id:
+            1549638
+        is_customer_case:
+            False
+        maintainer:
+            xiliang@redhat.com
+        description:
+            make sure there is no wheel in default_user's group in "/etc/cloud/cloud.cfg"
+        key_steps:
+            1. check cloud config file
+        expect_result:
+            there's no 'wheel' saved in log file
+        debug_want:
+            cloud.cfg
+        """
         cmd = 'sudo cat /etc/cloud/cloud.cfg'
         utils_lib.run_cmd(self,
                 cmd,
@@ -75,13 +186,19 @@ class TestAzureImage(unittest.TestCase):
 
     def test_check_blacklist(self):
         '''
-        rhbz: 1645772
-        nouveau,lbm-nouveau,floppy should be disabled
+        bz: 1645772
+        des: nouveau,lbm-nouveau,floppy,skx_edac,intel_cstate should be disabled
         '''
         utils_lib.run_cmd(self, "sudo lsmod|grep nouveau", expect_not_ret=0, msg='check nouveau is not loaded')
         file_check = '/lib/modprobe.d/blacklist-*.conf'
-        for module in ['nouveau, lbm-nouveau, floppy']:
-            utils_lib.run_cmd(self, "sudo cat {}".format(file_check), expect_ret=0, expect_kw='blacklist '+module, msg='check "{}" in {}'.format(module, file_check))
+        if self.rhel_x_version >= 9:
+            blacklist = ['nouveau', 'lbm-nouveau', 'floppy', 'amdgpu']
+        else:
+            blacklist = ['nouveau', 'lbm-nouveau', 'floppy', 'skx_edac', 'intel_cstate', 'amdgpu']
+        # Print the full blacklist
+        utils_lib.run_cmd(self, "sudo cat {}".format(file_check))
+        for module in blacklist:
+            utils_lib.run_cmd(self, "sudo cat {}|grep -w {}".format(file_check, module), expect_ret=0, msg='check "{}" in {}'.format(module, file_check))
 
     def test_check_cmdline_rhgb_quiet(self):
         '''
@@ -101,7 +218,7 @@ class TestAzureImage(unittest.TestCase):
 
     def test_check_pkg_wanted(self):
         '''
-        Some pkgs are required in azure.
+        Verify requied pkgs are installed.
         '''
         pkgs_wanted = '''yum-utils,redhat-release-eula,cloud-init,\
 tar,rsync,NetworkManager,cloud-utils-growpart,gdisk,\
@@ -130,7 +247,7 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
         - boot with efi or legacy bios
         - /boot/grub2/grubenv is a symlink for /boot/efi/EFI/redhat/grubenv if boot with efi
         - /boot/grub2/grubenv is a file rather than a link if boot with legacy bios
-        Check grub2 config for el8:
+        Check grub2 config for el8+:
         - boot with efi or legacy bios
         - /boot/grub2/grub.cfg exists and /boot/grub2/grubenv is a file if boot with efi
         - /boot/grub2/grubenv is a file rather than a link if boot with legacy bios
@@ -372,24 +489,17 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
         '''
         cmd = 'sudo cat /etc/yum/pluginconf.d/product-id.conf'
         utils_lib.run_cmd(self,cmd, expect_ret=0, expect_kw='enabled=1', msg='check yum product-id plugin is enabled')
-        cmd = "sudo rpm -qa|grep rhui"
-        ret = utils_lib.run_cmd(self, cmd, ret_status=True, msg='Check if it is a RHUI image')
-        if ret == 0:
-            # RHUI image
-            expect_kw = "enabled=0"
-            status = "disabled"
-        else:
-            # SCA image
-            expect_kw = "enabled=1"
-            status = "enabled"
-        if self.rhel_x_version >= 8:
-            expect_kw += ",disable_system_repos=0"
         # Yum plugin
+        yum_expect_kw = "enabled=0"
         cmd = "sudo cat /etc/yum/pluginconf.d/subscription-manager.conf|sed -s 's/ = /=/g'"
-        utils_lib.run_cmd(self,cmd, expect_ret=0, expect_kw=expect_kw, msg='check yum subscription-manager plugin is {}'.format(status))
+        utils_lib.run_cmd(self,cmd, expect_ret=0, expect_kw=yum_expect_kw, msg='check yum subscription-manager plugin is {}'.format(yum_expect_kw))
         # Dnf plugin
+        if self.rhel_x_version >= 8:
+            dnf_expect_kw = "enabled=0,disable_system_repos=0"
+        else:
+            dnf_expect_kw = "enabled=1"
         cmd = "sudo cat /etc/dnf/plugins/subscription-manager.conf|sed -s 's/ = /=/g'"
-        utils_lib.run_cmd(self,cmd, expect_ret=0, expect_kw=expect_kw, msg='check dnf subscription-manager plugin is {}'.format(status))
+        utils_lib.run_cmd(self,cmd, expect_ret=0, expect_kw=dnf_expect_kw, msg='check dnf subscription-manager plugin is {}'.format(dnf_expect_kw))
 
     def test_check_auditd(self):
         """
@@ -425,6 +535,7 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
 
     def test_check_yum_repoinfo(self):
         '''
+        Verify yum repoinfo, repo-pkgs should not be 0
         '''
         cmd = "sudo rpm -qa|grep rhui"
         ret = utils_lib.run_cmd(self, cmd, ret_status=True, msg='Check if it is a RHUI image')
@@ -435,6 +546,7 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
 
     def test_yum_package_install(self):
         '''
+        Verify yum function
         '''
         cmd = "sudo rpm -qa|grep rhui"
         utils_lib.run_cmd(self, cmd, expect_ret=0, msg='Check if RHUI is installed')
@@ -452,8 +564,8 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
         Verify auto_registration is enabled in the image
         '''
         product_id = utils_lib.get_product_id(self)
-        if float(product_id) < float('8.4'):
-            self.skipTest('skip in earlier than el8.4')
+        # if float(product_id) < float('8.4'):
+        #     self.skipTest('skip in earlier than el8.4')
 
         # cmd = "sudo rpm -qa|grep rhui"
         # ret = utils_lib.run_cmd(self, cmd, ret_status=True, msg='Check if it is a RHUI image')
@@ -471,7 +583,7 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
 
     # def test_check_pkgs(self):
     #     '''
-    #     Some pkgs are required in azure.
+    #     Compare full package list
     #     '''
     #     base_file = 'pkglist'
     #     test_file = '/tmp/pkglist_test'
@@ -491,7 +603,7 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
 
     def test_check_services_active(self):
         '''
-        Verify the services are active
+        Verify the necessary services are active
         '''
         services = '''waagent,cloud-init-local,cloud-init,cloud-config,cloud-final,hypervkvpd,sshd'''
         service_list = services.split(',')
@@ -513,16 +625,6 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
         cmd = "sudo kdumpctl status"
         utils_lib.run_cmd(self, cmd, expect_ret=0, expect_kw='Kdump is operational', msg='Check kdump status')
 
-    def _check_log(self, filename, ignore_list=[], keyword='err|fail|warn|trace'):
-        cmd = "sudo grep -iE 'err|fail|warn|trace' {}|grep -vE '{}'".format(filename, '|'.join(ignore_list))
-        output = utils_lib.run_cmd(self, cmd, msg="Check {} in {}".format(keyword, filename))
-        if output:
-            self.log.info("Unexpected logs exist in {}.\n{}".format(filename, output))
-            return False
-        else:
-            self.log.info("No unexpected logs in "+filename)
-            return True
-
     def test_check_messages(self):
         '''
         Verify no error/fail/trace in /var/log/messages
@@ -557,6 +659,28 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
         self._check_log('/tmp/dmesg.log', ignore_list)
 
     def test_check_cloudinit_log(self):
+        """
+        case_tag:
+            cloudinit
+        case_name:
+            test_check_cloudinit_log
+        component:
+            cloud-init
+        bugzilla_id:
+            N/A
+        is_customer_case:
+            False
+        maintainer:
+            xiliang@redhat
+        description:
+            verify no err/fail/warn/trace in /var/log/cloud-init.log
+        key_steps:
+            check if there's err/fail/warn/trace in cloud init log
+        expect_result:
+            no fail letter found
+        debug_want:
+            cloud init log file 
+        """
         '''
         Verify no err/fail/warn/trace in /var/log/cloud-init.log
         '''
@@ -570,7 +694,8 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
             'Stderr: mount: .*: unknown filesystem type .ntfs.',
             'SKIPPED: device_part_info.* failed: /dev/mapper/rootvg-rootlv not a partition',
             'modules with 0 failures',
-            'Failed to get raw userdata in module rightscale_userdata'
+            'Failed to get raw userdata in module rightscale_userdata',
+            'nofail'
         ]
         self._check_log('/var/log/cloud-init.log', ignore_list)
 
@@ -610,7 +735,7 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
         """
         check if there's boot time delay
         """
-        max_boot_time = 100
+        max_boot_time = 120
         boot_time_sec = utils_lib.getboottime(self)
         utils_lib.compare_nums(self, num1=boot_time_sec, num2=max_boot_time, ratio=0, msg="Compare with cfg specified max_boot_time")
 
@@ -619,7 +744,6 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
         Verify hyperv drivers are loaded
         '''
         hyperv_driver_list = [
-            'hyperv_fb',
             'hv_utils',
             'hv_balloon',
             'hv_storvsc',
@@ -629,6 +753,10 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
             'hyperv_keyboard',
             'hv_vmbus'
         ]
+        if self.rhel_x_version <= 8:
+            hyperv_driver_list.append('hyperv_fb')
+        else:
+            hyperv_driver_list.append('hyperv_drm')
         cmd = "/sbin/lsmod|grep -iE 'hv|hyperv'"
         utils_lib.run_cmd(self, cmd, expect_kw=','.join(hyperv_driver_list), msg="Verify hyperv drivers are loaded")
 
@@ -643,50 +771,51 @@ hypervkvpd,hyperv-daemons-license,hypervfcopyd,hypervvssd,hyperv-daemons'''
 
     def test_check_hostkey_permission(self):
         '''
+        bz: 2013644
         Verify /etc/ssh/ssh_host_xxx_key permission are 640, group is ssh_keys.
         '''
-        # BZ#2013644
         expected = "-rw-r-----.rootssh_keys"
         cmd = "ls -l /etc/ssh/{ssh_host_ecdsa_key,ssh_host_ed25519_key,ssh_host_rsa_key}|awk '{print $1$3$4}'|uniq"
         utils_lib.run_cmd(self, cmd, expect_output=expected, msg="Verify /etc/ssh/ssh_host_xxx_key permission is 640, group is ssh_keys")
 
     def test_check_kdump_configuration(self):
         '''
-        Check /etc/sysconfig/kdump and /etc/kdump.conf
+        Verify /etc/sysconfig/kdump and /etc/kdump.conf are not changed
         '''
-        if self.rhel_x_version < 8:
-            sysconfig_kdump = '''\
-KDUMP_COMMANDLINE=""
-KDUMP_COMMANDLINE_APPEND="irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 acpi_no_memhotplug transparent_hugepage=never nokaslr novmcoredd hest_disable"
-KDUMP_COMMANDLINE_REMOVE="hugepages hugepagesz slub_debug kaslr"
-KDUMP_IMG_EXT=""
-KDUMP_IMG="vmlinuz"
-KDUMP_KERNELVER=""
-KEXEC_ARGS=""
-'''
-            kdump_conf = '''\
-path /var/crash
-core_collector makedumpfile -l --message-level 1 -d 31
-'''
-        else:
-            sysconfig_kdump = '''\
-KDUMP_KERNELVER=""
-KDUMP_COMMANDLINE=""
-KDUMP_COMMANDLINE_REMOVE="hugepages hugepagesz slub_debug quiet log_buf_len swiotlb"
-KDUMP_COMMANDLINE_APPEND="irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 rootflags=nofail acpi_no_memhotplug transparent_hugepage=never nokaslr novmcoredd hest_disable"
-KEXEC_ARGS="-s"
-KDUMP_IMG="vmlinuz"
-KDUMP_IMG_EXT=""
-'''
-            kdump_conf = '''\
-path /var/crash
-core_collector makedumpfile -l --message-level 7 -d 31
-'''
-        # Check /etc/sysconfig/kdump
-        self._check_file_content(testfile="/etc/sysconfig/kdump", expected=sysconfig_kdump, msg="Check /etc/sysconfig/kdump")
+        self._check_file_not_changed("kexec-tools")
+#         if self.rhel_x_version < 8:
+#             sysconfig_kdump = '''\
+# KDUMP_COMMANDLINE=""
+# KDUMP_COMMANDLINE_APPEND="irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 acpi_no_memhotplug transparent_hugepage=never nokaslr novmcoredd hest_disable"
+# KDUMP_COMMANDLINE_REMOVE="hugepages hugepagesz slub_debug kaslr"
+# KDUMP_IMG_EXT=""
+# KDUMP_IMG="vmlinuz"
+# KDUMP_KERNELVER=""
+# KEXEC_ARGS=""
+# '''
+#             kdump_conf = '''\
+# path /var/crash
+# core_collector makedumpfile -l --message-level 1 -d 31
+# '''
+#         else:
+#             sysconfig_kdump = '''\
+# KDUMP_KERNELVER=""
+# KDUMP_COMMANDLINE=""
+# KDUMP_COMMANDLINE_REMOVE="hugepages hugepagesz slub_debug quiet log_buf_len swiotlb"
+# KDUMP_COMMANDLINE_APPEND="irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 rootflags=nofail acpi_no_memhotplug transparent_hugepage=never nokaslr novmcoredd hest_disable"
+# KEXEC_ARGS="-s"
+# KDUMP_IMG="vmlinuz"
+# KDUMP_IMG_EXT=""
+# '''
+#             kdump_conf = '''\
+# path /var/crash
+# core_collector makedumpfile -l --message-level 7 -d 31
+# '''
+#         # Check /etc/sysconfig/kdump
+#         self._check_file_content(testfile="/etc/sysconfig/kdump", expected=sysconfig_kdump, msg="Check /etc/sysconfig/kdump")
         
-        # Check /etc/kdump.conf
-        self._check_file_content(testfile="/etc/kdump.conf", expected=kdump_conf, msg="Check /etc/kdump.conf")
+#         # Check /etc/kdump.conf
+#         self._check_file_content(testfile="/etc/kdump.conf", expected=kdump_conf, msg="Check /etc/kdump.conf")
 
     def test_check_dnf_conf(self):
         '''
@@ -739,42 +868,6 @@ skip_if_unavailable=False
         cmd = "sudo grep ^ClientAliveInterval /etc/ssh/sshd_config"
         utils_lib.run_cmd(self, cmd, expect_output="ClientAliveInterval 180", msg="Verify ClientAliveInterval is 180")
 
-    def _check_file_content(self, basefile=None, testfile=None, expected=None, msg=None, project=None):
-        '''
-        Function to check file content.
-        '''
-        # Print testfile content for recording
-        utils_lib.run_cmd(self, "cat "+testfile)
-        # If base is a file
-        if basefile:
-            # Base file content is different in multiple RHEL versions
-            if project:
-                basefile += "_rhel{}".format(project)
-            # Get the base file path
-            # If remote node, copy the basefile to the remote node /tmp/
-            src_file = self.data_dir + '/azure/{}'.format(basefile)
-            if self.params['remote_node'] is not None:
-                base_file = "/tmp/"+basefile
-                self.log.info('Copy {} to remote'.format(basefile))
-                self.SSH.put_file(local_file=src_file, rmt_file=base_file)
-            else:
-                base_file = src_file
-            test_file = testfile
-        # If base is a block of strings
-        elif expected:
-            base_file = '/tmp/basefile'
-            utils_lib.run_cmd(self, "echo '''{0}''' > {1};sort {1}>{2}".format(expected, '/tmp/tmpfile', base_file))
-            test_file = '/tmp/testfile'
-            cmd = "sudo cat {}|grep -v '^#'|sed '/^$/d'|sort > {}".format(testfile, test_file)
-            utils_lib.run_cmd(self, cmd)
-        else:
-            self.error('Must specify basefile or expected!')
-        if not msg:
-            msg = "Check "+testfile
-        # Compare files ignore space, empty line and order
-        output = utils_lib.run_cmd(self, "sudo diff -wB {} {}".format(base_file, test_file), msg=msg)
-        self.assertEqual(output, '', "{} and {} are not the same.\n{}".format(base_file, test_file, output))
-
     def test_check_91_azure_datasource(self):
         '''
         Check file /etc/cloud/cloud.cfg.d/91-azure_datasource.cfg
@@ -809,10 +902,14 @@ skip_if_unavailable=False
 
     def test_check_cloud_cfg(self):
         '''
-        Check file /etc/cloud/cloud.cfg
+        Verify file /etc/cloud/cloud.cfg is not changed
+        (RHEL-9 only) Verify _netdev is in cloud.cfg
         '''
-        filename = '/etc/cloud/cloud.cfg'
-        self._check_file_content(filename.split('/')[-1], filename, project=self.rhel_x_version)
+        # filename = '/etc/cloud/cloud.cfg'
+        # self._check_file_content(filename.split('/')[-1], filename, project=self.rhel_x_version)
+        self._check_file_not_changed("cloud-init")
+        if self.rhel_x_version >= 9:
+            utils_lib.run_cmd(self, cmd="cat /etc/cloud/cloud.cfg", expect_kw="_netdev", msg="Verify _netdev is in RHEL-9 cloud.cfg")
 
     def test_check_pwquality_conf(self):
         '''
@@ -897,10 +994,11 @@ plugins = ifcfg-rh,
         
     def test_check_chrony_conf(self):
         '''
-        Check file /etc/chrony.conf
+        Verify file /etc/chrony.conf is not changed
         '''
-        filename = '/etc/chrony.conf'
-        self._check_file_content(filename.split('/')[-1], filename, project=self.rhel_x_version)
+        # filename = '/etc/chrony.conf'
+        # self._check_file_content(filename.split('/')[-1], filename, project=self.rhel_x_version)
+        self._check_file_not_changed("chrony")
 
     def test_check_authconfig(self):
         '''
@@ -978,8 +1076,11 @@ langpack_locales = en_US.UTF-8
         '''
         Verify os disk size is 64 GiB
         '''
-        cmd = "sudo fdisk -l|grep 'Disk /dev/sda'|awk '{print $5}'"
-        utils_lib.run_cmd(self, cmd, expect_kw='68719476736', msg="Verify os disk size is 64 GiB")
+        if self.rhel_x_version < 8:
+            cmd = "sudo fdisk -l|grep 'Linux LVM'|awk '{print $4}'"
+        else:
+            cmd = "sudo fdisk -l|grep 'Linux LVM'|awk '{print $5}'"
+        utils_lib.run_cmd(self, cmd, expect_kw='63G', msg="Verify os disk size is 64 GiB")
 
     # Inactive this case because the service list is not always the same
     # def test_check_service_list(self):
@@ -994,9 +1095,9 @@ langpack_locales = en_US.UTF-8
 
     def test_check_sshd_config(self):
         '''
-        Check file /etc/ssh/sshd_config
+        Check file content /etc/ssh/sshd_config
         '''
-        if self.rhel_x_version < 8:
+        if self.rhel_x_version == 7:
             expected = '''\
 AcceptEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
 AcceptEnv LC_IDENTIFICATION LC_ALL LANGUAGE
@@ -1016,7 +1117,7 @@ SyslogFacility AUTHPRIV
 UsePAM yes
 X11Forwarding yes
 '''
-        else:
+        elif self.rhel_x_version == 8:
             expected = '''\
 AcceptEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
 AcceptEnv LC_IDENTIFICATION LC_ALL LANGUAGE
@@ -1037,6 +1138,14 @@ Subsystem sftp	/usr/libexec/openssh/sftp-server
 SyslogFacility AUTHPRIV
 UsePAM yes
 X11Forwarding yes
+'''
+        else:
+            expected = '''\
+Include /etc/ssh/sshd_config.d/*.conf
+AuthorizedKeysFile .ssh/authorized_keys
+Subsystem sftp	/usr/libexec/openssh/sftp-server
+ClientAliveInterval 180
+PasswordAuthentication no
 '''
         filename = "/etc/ssh/sshd_config"
         self._check_file_content(testfile=filename, expected=expected)
@@ -1045,10 +1154,7 @@ X11Forwarding yes
         '''
         Verify can get metadata
         '''
-        cmd = 'curl -s -H Metadata:true --noproxy "*" "http://169.254.169.254/metadata/instance?api-version=2021-02-01"'
-        metadata = utils_lib.run_cmd(self, cmd)
-        metadata_dict = json.loads(metadata)
-        self.assertEqual(metadata_dict.get('compute').get('azEnvironment'), "AzurePublicCloud", 
+        self.assertEqual(self.metadata.get('compute').get('osType'), "Linux", 
             "Cannot parse metadata to get azEnvironment")
 
     def test_check_cds_hostnames(self):
@@ -1059,25 +1165,25 @@ X11Forwarding yes
             "rhui-1.microsoft.com",
             "rhui-2.microsoft.com",
             "rhui-3.microsoft.com",
-            "{0}-cds.{0}.cloudapp.azure.com".format(self.region),
-            "rh-cds.trafficmanager.net"
         ]
         for cds in rhui_cds_hostnames:
-            #there is no rhui in us-gov regions at all - all the content requests are redirected to closest standard regions
-            cds_name = cds.replace('-gov','')
-            cmd = "sudo getent hosts {}".format(cds_name)
-            utils_lib.run_cmd(self, cmd, expect_ret=0, msg='check {}'.format(cds_name))
+            cmd = "sudo getent hosts {}".format(cds)
+            utils_lib.run_cmd(self, cmd, expect_ret=0, msg='check {}'.format(cds))
 
     def test_z_check_subscription_manager_auto_function(self):
         '''
-        * Add "z" in the case name to make it run at last
         Verify auto_registration function works
+        * Add "z" in the case name to make it run at last
         '''
         product_id = utils_lib.get_product_id(self)
-        if float(product_id) < float('8.4'):
-            self.skipTest('skip in earlier than el8.4')
+        # if float(product_id) < float('8.4'):
+        #     self.skipTest('skip in earlier than el8.4')
+        cmd = "sudo subscription-manager config --rhsmcertd.auto_registration=1"
+        utils_lib.run_cmd(self, cmd, expect_ret=0, msg="Enable auto_registration")
+        cmd = "sudo subscription-manager config --rhsm.manage_repo=0"
+        utils_lib.run_cmd(self, cmd, expect_ret=0, msg="Disable manage_repo")
         cmd = "sudo subscription-manager config --rhsmcertd.auto_registration_interval=1"
-        utils_lib.run_cmd(self, cmd, expect_ret=0, msg="Check interval to 1 min")
+        utils_lib.run_cmd(self, cmd, expect_ret=0, msg="Change interval to 1 min")
         cmd = "sudo systemctl restart rhsmcertd.service"
         utils_lib.run_cmd(self, cmd, expect_ret=0, msg="Restart rhsmcertd service")
         time.sleep(60)
@@ -1092,6 +1198,18 @@ X11Forwarding yes
         else:
             self.fail("Fail to auto register!")
 
+    def test_check_image_generation(self):
+        '''
+        (image test only)Check generation according to image name
+        '''
+        sku = self.metadata['compute']['storageProfile']['imageReference']['sku']
+        if not sku:
+            self.skipTest("Only for image testing")
+        if sku.endswith('gen1'):
+            self.assertEqual(self._get_generation(), 'gen1', "Expected: gen1; Real: gen2")
+        else:
+            self.assertEqual(self._get_generation(), 'gen2', "Expected: gen2; Real: gen1")
+
     def tearDown(self):
         pass
 
