@@ -42,45 +42,6 @@ class TestGeneralCheck(unittest.TestCase):
             cmd = 'sudo bash -c "{} {}"'.format(check_file, self.systemd_analyze_verify_file)
             output = utils_lib.run_cmd(self, cmd, timeout=240, msg="start analyze......")
 
-    def test_check_trigger_automount(self):
-        """
-        case_name:
-            test_check_trigger_automount
-        component:
-            sysstat
-        bugzilla_id:
-            1670060
-        is_customer_case:
-            True
-        maintainer:
-            xuazhao@redhat.com
-        description:
-            check sysstat not triggering a mount of all automount filesystem and statistic gather even when not selected.
-        key_steps:
-            1. sudo yum install -y sysstat strace
-            2. rm -rf sa.new
-            3. strace -fttT -o sadc.trc /usr/lib64/sa/sadc -F -L 1 2 sa.new
-            4. grep mtab sadc.trc
-        expect_result:
-            no mstab keyword found,eg.17234 13:38:38.058270 open("/etc/mtab", O_RDONLY) = 3 <0.000428>
-        debug_want:
-            version of sysstat
-        """
-        
-        utils_lib.is_pkg_installed(self,'sysstat strace', cancel_case=True)
-
-        cmd = 'rpm -q sysstat'
-        utils_lib.run_cmd(self,cmd)
-
-        cmd = 'rm -rf sa.new'
-        utils_lib.run_cmd(self,cmd,msg='clean old data')
-
-        cmd = 'strace -fttT -o sadc.trc /usr/lib64/sa/sadc -F -L 1 2 sa.new'
-        utils_lib.run_cmd(self,cmd,msg='generate report')
-
-        cmd = 'grep mtab sadc.trc'
-        utils_lib.run_cmd(self,cmd, expect_not_ret=0, msg='check ret')
-
     def test_check_avclog(self):
         """
         case_name:
@@ -445,9 +406,9 @@ available_clocksource'
             cmd = "sudo dmidecode --dump-bin {}".format(binfile)
             utils_lib.run_cmd(self, cmd, msg='save dmidecode_debug.bin for debug purpose, please attach it if file bug')
             self.log.info('retrive {} from remote'.format(binfile))
-            self.SSH.get_file(rmt_file=binfile,local_file='{}/debug/dmidecode_debug.bin'.format(self.log_dir))
+            self.SSH.get_file(rmt_file=binfile,local_file='{}/attachments/dmidecode_debug.bin'.format(self.log_dir))
         else:
-            cmd = "sudo dmidecode --dump-bin {}/debug/dmidecode_debug.bin".format(self.log_dir)
+            cmd = "sudo dmidecode --dump-bin {}/attachments/dmidecode_debug.bin".format(self.log_dir)
             utils_lib.run_cmd(self, cmd, msg='save dmidecode_debug.bin for debug purpose, please attach it if file bug')
         cmd = "sudo dmidecode --dump"
         utils_lib.run_cmd(self, cmd, expect_ret=0, expect_not_kw='Segmentation')
@@ -467,7 +428,7 @@ available_clocksource'
         description:
             dmidecode command prints "OUT OF SPEC" messages for empty NVMe and DIMM slots
         key_steps:
-            1.sudo dmidecode --dump-bin {log_dir}/debug/dmidecode_debug.bin
+            1.sudo dmidecode --dump-bin {log_dir}/attachments/dmidecode_debug.bin
             2.sudo dmidecode
         expect_result:
             It should not print "OUT OF SPEC" in output
@@ -475,7 +436,7 @@ available_clocksource'
             dmidecode_debug.bin
         """
         utils_lib.is_cmd_exist(self, cmd='dmidecode')
-        cmd = "sudo dmidecode --dump-bin {}/debug/dmidecode_debug.bin".format(self.log_dir)
+        cmd = "sudo dmidecode --dump-bin {}/attachments/dmidecode_debug.bin".format(self.log_dir)
         utils_lib.run_cmd(self, cmd, msg='save dmidecode_debug.bin for debug purpose, please attach it if file bug')
         utils_lib.run_cmd(self,
                     'sudo dmidecode',
@@ -513,7 +474,7 @@ available_clocksource'
         utils_lib.run_cmd(self, "rpm -qa|grep linux-firmware",msg='get linux-firmware pkg version')
         check_cmd = r"sudo grep . /sys/devices/system/cpu/vulnerabilities/* | \
 sed 's/:/^/' | column -t -s^"
-        utils_lib.run_cmd(self, check_cmd, expect_ret=0)
+        utils_lib.run_cmd(self, check_cmd, expect_ret=0, msg='retrive cpu vulnerabilities statics')
 
         output = utils_lib.run_cmd(self, 'uname -r', expect_ret=0)
         if utils_lib.is_metal(self):
@@ -526,13 +487,13 @@ sed 's/:/^/' | column -t -s^"
                 "Skip spec_store_bypass,Retpoline and mds in RHEL7 vms")
             check_cmd = r"sudo grep . /sys/devices/system/cpu/vulnerabilities/* | \
 grep -v spec_store_bypass|grep -v 'tsx_async_abort'|grep -v 'Vulnerable: Retpoline'|\
-grep -v mds| sed 's/:/^/' | column -t -s^"
+grep -v mds|grep -v 'no microcode'|grep -v retbleed| sed 's/:/^/' | column -t -s^"
         else:
             self.log.info(
                 "Skip spec_store_bypass and mds,itlb_multihit in vms")
             check_cmd = r"sudo grep . /sys/devices/system/cpu/vulnerabilities/*|\
 grep -v spec_store_bypass|grep -v 'tsx_async_abort'|grep -v mds|grep -v \
-itlb_multihit|sed 's/:/^/' | column -t -s^"
+itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -s^"
 
         utils_lib.run_cmd(self, check_cmd, expect_ret=0, expect_not_kw='Vulnerable')
 
@@ -829,9 +790,9 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
                 utils_lib.run_cmd(self, cmd, expect_ret=0)
                 if self.params['remote_node'] is not None:
                     self.log.info('retrive {} from remote to {}'.format(core_file, self.log_dir))
-                    self.SSH.get_file(rmt_file=core_file,local_file='{}/debug/{}'.format(self.log_dir,os.path.basename(core_file)))
+                    self.SSH.get_file(rmt_file=core_file,local_file='{}/attachments/{}'.format(self.log_dir,os.path.basename(core_file)))
                 else:
-                    cmd = "cp {} {}/debug/{}".format(core_file, self.log_dir,os.path.basename(core_file) )
+                    cmd = "cp {} {}/attachments/{}".format(core_file, self.log_dir,os.path.basename(core_file) )
                     utils_lib.run_cmd(self, cmd, msg='save {} to {}'.format(core_file, self.log_dir))
         utils_lib.check_log(self, 'dumped core', skip_words='test_check_journalctl_dumpedcore', rmt_redirect_stdout=True)
 
@@ -997,7 +958,7 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
         component:
             systemd
         bugzilla_id:
-            1871139
+            1871139,2115230
         customer_case_id:
             yes
         testplan:
@@ -1045,6 +1006,20 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
         cmd = 'rpm -qa | grep glibc'
         utils_lib.run_cmd(self, cmd, msg='please attach {} output if file bug'.format(cmd))
         utils_lib.run_cmd(self, 'locale', expect_not_kw="Cannot", msg='check no errors about locale')
+        cmd = "grep 'LANG=' /etc/locale.conf"
+        output = utils_lib.run_cmd(self, cmd, expect_ret=0, msg="Get LANG in locale.conf")
+        product_id = utils_lib.get_product_id(self)
+        if float(product_id) >= 9.0:
+            # workaround for the pending pkg release
+            cmd = "rpm -q cloud-init"
+            output = utils_lib.run_cmd(self, cmd, expect_ret=0, msg="Get cloud-init rpm name")
+            ver = output.lstrip('cloud-init-').split('-')[0]
+            if (float(ver) >= 22.2):
+                self.assertTrue(re.match(r'LANG="*C.UTF-8"*', output),
+                                "default LANG is wrong -> %s" % output)
+        else:
+            self.assertTrue(re.match(r'LANG="*en_US.UTF-8"*', output),
+                            "default LANG is wrong -> %s" % output)
 
     def test_check_lshw_mem(self):
         '''
@@ -1084,7 +1059,10 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
         mem_in_gib = mem_in_byte/1024/1024/1024
         self.log.info("lshw showed mem: {}".format(mem_in_gib))
 
-        utils_lib.compare_nums(self, mem_in_gib, base_memory, ratio=15)
+        if mem_in_gib >= 4:
+            utils_lib.compare_nums(self, mem_in_gib, base_memory, ratio=15)
+        else:
+            utils_lib.compare_nums(self, mem_in_gib, base_memory, ratio=17)
 
     def test_check_lsmem_segfault(self):
         '''
@@ -1831,8 +1809,10 @@ current_device"
             self.assertEqual('aws', virt_what_output.strip('\n'))
         elif utils_lib.is_metal(self):
             self.log.info("Found it is a bare metal system!")
-        elif utils_lib.is_aws(self) and utils_lib.is_arch(self, arch='aarch64'):
+        elif utils_lib.is_aws(self) and utils_lib.is_arch(self, arch='aarch64') and utils_lib.is_metal(self):
             self.assertEqual('aws', virt_what_output.strip('\n'))
+        elif utils_lib.is_aws(self) and utils_lib.is_arch(self, arch='aarch64') and not utils_lib.is_metal(self):
+            self.assertEqual('kvm\naws\n', virt_what_output)
         else:
             self.skipTest("Unknow hypervisor")
 
@@ -2011,9 +1991,9 @@ current_device"
                 msg="please attach this archive if file bug", timeout=180)
         gz_file = re.findall('/var/.*tar.gz', out)[0]
         file_name = gz_file.split('/')[-1]
-        if self.params['remote_node'] is not None:
+        if self.params.get('remote_node') is not None:
             self.log.info('retrive {} from remote'.format(file_name))
-            self.SSH.get_file(rmt_file='/tmp/{}'.format(file_name),local_file='{}/debug/{}'.format(self.log_dir,file_name))
+            self.SSH.get_file(rmt_file='/tmp/{}'.format(file_name),local_file='{}/attachments/{}'.format(self.log_dir,file_name))
         else:
             utils_lib.run_cmd(self, 'sudo cp {} {}'.format(gz_file, self.log_dir))
         try:
@@ -2022,6 +2002,7 @@ current_device"
                 self.fail("{} insights rule hit".format(len(tmp_dict)))
         except json.decoder.JSONDecodeError as exc:
             self.log.error("insights rule hit or other unexpected error")
+
     def test_check_sos_works(self):
         """
         case_name:
@@ -2055,10 +2036,81 @@ current_device"
         utils_lib.run_cmd(self, cmd, expect_ret=0)
         if self.params['remote_node'] is not None:
             self.log.info('retrive {} from remote to {}'.format(sosfile, self.log_dir))
-            self.SSH.get_file(rmt_file=sosfile,local_file='{}/debug/{}'.format(self.log_dir,os.path.basename(sosfile)))
+            self.SSH.get_file(rmt_file=sosfile,local_file='{}/attachments/{}'.format(self.log_dir,os.path.basename(sosfile)))
         else:
-            cmd = "cp {} {}/debug/{}".format(sosfile, self.log_dir,os.path.basename(sosfile) )
+            cmd = "cp {} {}/attachments/{}".format(sosfile, self.log_dir,os.path.basename(sosfile) )
             utils_lib.run_cmd(self, cmd, msg='save {} to {}'.format(sosfile, self.log_dir))
+
+    def test_check_dmesg_sev(self):
+        """
+        case_name:
+            test_check_dmesg_sev
+        case_file:
+            os_tests.tests.test_general_check.TestGeneralCheck.test_check_dmesg_sev
+        component:
+            kernel
+        bugzilla_id:
+            2103821
+        customer_case_id:
+            False
+        testplan:
+            N/A
+        maintainer:
+            wshi@redhat.com
+        description:
+            Make sure there is SEV keyword from dmesg output.
+        key_steps:
+            # dmesg|grep -i sev
+        expect_result:
+            AMD Memory Encryption Features active: SEV
+        debug_want:
+            # dmesg
+        """
+        try:
+            if utils_lib.is_sev_enabled(self):
+                utils_lib.run_cmd(self, 'dmesg', expect_ret=0,
+                                  expect_kw='AMD Memory Encryption Features active: SEV',
+                                  msg="Check there is 'AMD Memory Encryption Features active: SEV' in dmesg before run 'perf top'")
+            else:
+                self.skipTest('SEV is not enabled')
+        except NotImplementedError:
+                self.skipTest('SEV check is not implemented on %s' % self.vm.provider)
+
+    def test_check_secure_ioerror(self):
+        """
+        case_tag:
+            secure_log
+        case_name:
+            test_check_secure_ioerror
+        case_file:
+            os_tests.tests.test_general_check.test_check_secure_ioerror
+        component:
+            secure_log
+        bugzilla_id:
+            1103344
+        is_customer_case:
+            False
+        customer_case_id:
+            N/A
+        testplan:
+            N/A
+        maintainer:
+            shshang@redhat.com
+        description: |
+            Check there is no io error in /var/log/secure.
+            https://access.redhat.com/solutions/975803
+        key_steps:
+            1. Check /var/log/secure via command "sudo cat /var/log/secure".
+        expect_result:
+            There isn't "Input/output error" in secure log.
+        debug_want:
+            N/A
+        """
+        self.log.info("Check /var/log/secure")
+        utils_lib.run_cmd(self, "sudo cat /var/log/secure", expect_ret=0)
+        utils_lib.run_cmd(self, "sudo cp /var/log/secure /tmp", expect_ret=0)
+        utils_lib.run_cmd(self, "sudo cat /var/log/secure", expect_not_kw="Input/output error")
+
     def tearDown(self):
         pass
 
