@@ -367,8 +367,11 @@ int main(int argc, char *argv[])
             10.sudo insights-client --register
         expect_result:
             all registration successfully without timeout failure.
-        debug_want:
-            N/A
+        debug_want: |
+            If you linked the test account to customer portal bug auto-reg failed,
+            Please attach below log if there is new region added in aws.
+            $ curl -s http://169.254.169.254/latest/dynamic/instance-identity/rsa2048
+            $ curl -s http://169.254.169.254/latest/dynamic/instance-identity/document
         """
         product_name = utils_lib.get_os_release_info(self, field='NAME')
         if 'Red Hat Enterprise Linux' not in product_name:
@@ -379,6 +382,10 @@ int main(int argc, char *argv[])
         if float(product_id) < 8.4:
             self.skipTest('skip in earlier than el8.4')
 
+        if utils_lib.is_aws(self):
+            cmds = ['curl -s http://169.254.169.254/latest/dynamic/instance-identity/rsa2048','curl -s http://169.254.169.254/latest/dynamic/instance-identity/document']
+            for cmd in cmds:
+                utils_lib.run_cmd(self, cmd, msg='region identity data')
         cmd = "sudo subscription-manager config --rhsmcertd.auto_registration=1 --rhsm.manage_repos=0 --rhsmcertd.auto_registration_interval=1"
         utils_lib.run_cmd(self, cmd, expect_ret=0, msg='try to enable auto_registration, disable managed_repos and change inverval from 60mins to 1min')
         cmd = "sudo systemctl restart rhsmcertd"
@@ -511,7 +518,7 @@ int main(int argc, char *argv[])
         utils_lib.run_cmd(self, cmd, expect_ret=0)
         utils_lib.run_cmd(self, 'ulimit -c unlimited;/tmp/pp', msg='run it to trigger core dump')
         utils_lib.run_cmd(self, 'sudo ls /var/lib/systemd/coredump/core.pp*', expect_ret=0, msg='check core file generated')
-        utils_lib.check_log(self, "error,warn,fail", log_cmd='journalctl -b0', cursor=self.cursor, rmt_redirect_stdout=True)
+        utils_lib.check_log(self, "warn,fail", log_cmd='journalctl -b0', cursor=self.cursor, rmt_redirect_stdout=True)
 
     def test_podman_build_image(self):
         '''
@@ -725,13 +732,10 @@ RUN touch /tmp/test.txt
                 Device pci_0000_2b_00_0 re-attached
         '''
         utils_lib.is_metal(self, action="cancel")
-        cmd = "sudo yum install -y libvirt"
-        utils_lib.run_cmd(self, cmd, msg = "install libvirt pkg")
+        utils_lib.is_pkg_installed(self, pkg_name="libvirt")
         cmd = "sudo systemctl restart libvirtd"
         utils_lib.run_cmd(self, cmd, cancel_ret='0', msg = "restart libvirtd")
         utils_lib.is_cmd_exist(self, cmd='virsh')
-        self.ssh_timeout = 1200
-        self.log.info('set ssh connection timeout to {}'.format(self.ssh_timeout))
         if utils_lib.is_arch(self, arch='x86_64'):
             boot_param_required = 'intel_iommu=on'
             out = utils_lib.run_cmd(self, 'cat /proc/cmdline', msg='Check boot line')
@@ -888,13 +892,15 @@ if __name__ == "__main__":
             "strace -o log wipefs -a /dev/srN
         """
         cmd = "lsblk | grep sr"
-        all = utils_lib.run_cmd(self,cmd,cancel_kw="sr0",msg="check if machine mounted CDROM").split("\n")
+        all = utils_lib.run_cmd(self,cmd,cancel_kw="sr0",
+                                msg="check if machine mounted CDROM").rstrip().split("\n")
+        self.cursor = utils_lib.get_cmd_cursor(self, cmd='journalctl -b0', rmt_redirect_stdout=True)
         for i in all:
             part = i.split(" ")[0]
-            cmd = "wipefs -a /dev/"+part
+            cmd = "sudo wipefs -a /dev/"+part
             utils_lib.run_cmd(self,cmd,expect_ret=1,msg="erase signature")
-            cmd = "dmesg -T | grep error"
-            utils_lib.run_cmd(self,cmd,expect_not_kw=part,msg="check if there's error")
+            cmd = "dmesg -T | grep %s" % part
+            utils_lib.check_log(self, "error", log_cmd=cmd, cursor=self.cursor, rmt_redirect_stdout=True)
 
     def test_grub2_mkconfig(self):
         """
@@ -920,12 +926,18 @@ if __name__ == "__main__":
         debug_want:
             grub.cfg
         """
-        cmd='rpm -qa "grub2*"'
+        cmd = 'rpm -qa "grub2*"'
         utils_lib.run_cmd(self,cmd,msg="show grub version")
-        cmd="grub2-mkconfig -o /tmp/grub.cfg"
+        cmd = 'grub2-mkconfig -h'
+        out = utils_lib.run_cmd(self,cmd,msg="check if it has no-grubenv-update support")
+        if 'no-grubenv-update' in out:
+            cmd = "sudo grub2-mkconfig -o /tmp/grub.cfg --no-grubenv-update"
+        else:
+            cmd = "sudo grub2-mkconfig -o /tmp/grub.cfg"
         utils_lib.run_cmd(self,cmd,expect_kw="done",msg="generate file")
-        cmd='cat /tmp/grub.cfg'
+        cmd = 'sudo cat /tmp/grub.cfg'
         utils_lib.run_cmd(self,cmd,msg="check if there is saved_entry",expect_kw="{saved_entry"+"}")
+
     def test_z_nitro_enclaves(self):
         '''
         case_name:
